/**
 * Copyright (C) All rights reserved.  
 *
 * @Module     ArgumentInputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/08/09
 *             2.1 2010/08/30 - 2.1 2010/10/08
 *             2.1 2010/10/20 add setLogStream() and logStream variable
 *             2.1 2013/06/07 Modify for reading user class object.
 *
 * This class provides features for reading parameters.
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;

import oserv.*;

/**
 * Define an interface for reading operation
 */
interface ioReadOperator {

    public void setDebugLogStream(int level, LogPrintStream ps) ;
    public void readOperation(RetryObjectInputStream ais) throws Exception;
    public void setMessageResource(MessageResource msgrs) ;
    public void setPersistentPath(String path);
    public String getPersistentPath();
    public MessageResource getMessageResource() ;
}

class baseReadOperator  {
    boolean flag = false;
    protected static int debug = 0;
    protected LogPrintStream logStream = null;
    protected MessageResource message_resource = new MessageResource();
    protected String persistent_path = null;

    public void setDebugLogStream(int level, LogPrintStream ps) {
        if (level > debug) {
            debug = level;
        }
        if (ps != null) { 
            logStream = ps;
        }
    }

    public int getDebug() {
        return debug;
    }

    public LogPrintStream getLogStream() {
        return logStream;
    }

    protected Exception getException() {
        Exception local_exception_object;
        local_exception_object = new Exception(
           getMessageResource().getFormattedMessage(
           ObjectServerException.NO_OPERATION_EXCEPTION,
           getMessageResource().getMessage(
               ObjectServerException.NO_OPERATION_EXCEPTION)));
        return local_exception_object;
    }

    public void setFlag (boolean value) {
        flag = value;
    }
    protected boolean checkFlag() {
        return flag;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setPersistentPath(String path) {
        persistent_path = path;
    }

    public String getPersistentPath() {
        return persistent_path;
    }
}

/**
 * For native int type, this calls getintValue().
 */
class intReadOperator extends baseReadOperator implements ioReadOperator {
    int  iop;

    public int getintValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        int.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(iop)));
            }
        }
        return iop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            iop = network_object.getInt();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            int.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        int.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(iop)));
            }
        }
        return ;
    }
}

/**
 * For native long type, this calls readLong().
 */
class longReadOperator extends baseReadOperator implements ioReadOperator {
    long  lop;

    public long getlongValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        long.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(lop)));
            }
        }
        return lop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        Long  long_value;
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            lop = network_object.getLong();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            long.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        long.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(lop)));
            }
        }
        return ;
    }
}

/**
 * For native float type, this calls readFloat().
 */
class floatReadOperator extends baseReadOperator implements ioReadOperator {
    float  fop;

    public float getfloatValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        float.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(fop)));
            }
        }
        return fop;
    }

    public void readOperation(RetryObjectInputStream 
        retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            fop = network_object.getFloat();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            float.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.READ_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.READ_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    float.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(fop)));
            }
        }
        return ;
    }
}

/**
 * For native double type, this calls readDouble().
 */
class doubleReadOperator extends baseReadOperator implements ioReadOperator {
    double  dop;

    public double getdoubleValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        double.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(dop)));
            }
        }
        return dop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            dop = network_object.getDouble();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln( getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            double.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        double.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(dop)));
            }
        }
        return ;
    }
}

/**
 * For native boolean type, this calls readBoolean().
 */
class booleanReadOperator extends baseReadOperator implements ioReadOperator {
    boolean  bop;

    public boolean getbooleanValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        boolean.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(bop)));
            }
        }
        return bop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            bop = network_object.getBoolean();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            boolean.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        boolean.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(bop)));
            }
        }
        return ;
    }
}

class byteReadOperator extends baseReadOperator implements ioReadOperator {
    byte  byop;

    public byte getbyteValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        byte.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(byop)));
            }
        }
        return byop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            byop = network_object.getByte();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            byte.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        byte.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(byop)));
            }
        }
        return ;
    }
}

/**
 * For native char type, this calls readChar().
 */
class charReadOperator extends baseReadOperator implements ioReadOperator {
    char  cop;

    public char getcharValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        char.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(cop)));
            }
        }
        return cop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            cop = network_object.getChar();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            char.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        char.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(cop)));
            }
        }
        return ;
    }
}

/**
 * For native short type, this calls readShort().
 */
class shortReadOperator extends baseReadOperator implements ioReadOperator {
    short  sop;

    public short getshortValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        short.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(sop)));
            }
        }
        return sop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject   network_object;
        try {
            network_object = (NetworkObject)retry_object_input_stream.retryReadObject();
            sop = network_object.getShort();
            setFlag(true);
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            short.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        short.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(sop)));
            }
        }
        return ;
    }
}

/**
 * For native Object type, this calls readObject().
 */
class objectReadOperator extends baseReadOperator implements ioReadOperator {
    Object  oop;
    String  parameter_type;
    int     parameter_index_number;
    private static String ARRAY_BYTE_CLASS_NAME = "[B";

    protected Exception getParameterException() {
        Exception local_exception_object;

        local_exception_object = new Exception(
           getMessageResource().getFormattedMessage(
           ObjectServerException.PARAMETER_ERROR_EXCEPTION,
           getMessageResource().getMessage(
               ObjectServerException.PARAMETER_ERROR_EXCEPTION)));
        return local_exception_object;
    }

    public void setParameterType(String type) {
        parameter_type = type;
    }

    public void setParameterIndex(int index) {
        parameter_index_number = index;
    }

    public Object getObjectValue() throws Exception {
        
        if (checkFlag() == false) {
           Exception  exception_object ;
           exception_object = getException();
           throw exception_object; 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.EXTRACT_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.EXTRACT_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.getClass().getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY));
            }
        }
        return oop;
    }

    public void readOperation(
        RetryObjectInputStream retry_object_input_stream) throws Exception {
        NetworkObject  network_object;
        Object         return_object;
        String object_type;
        int parameter_index;
        byte[]                     array;
        File                       file_object;
        File                       file_descriptor;
        ByteArrayOutputStream      internal_byte_output_stream;
        ByteArrayInputStream       internal_byte_input_stream;
        FileOutputStream           internal_file_output_stream;
        ObjectOutputStream         internal_object_output_stream = null;
        NamedPipedInputStream      internal_named_piped_input_stream;
        NamedPipedOutputStream     internal_named_piped_output_stream;
        NamedFileOutputStream      internal_named_file_output_stream;
        NamedObjectOutputStream    internal_named_object_output_stream;
        NamedDataOutputStream      internal_named_data_output_stream;
        NamedBufferedOutputStream  internal_named_buffered_output_stream;
        NamedFilterOutputStream    internal_named_filter_output_stream;
        NamedPrintStream           internal_named_print_stream;
        NamedFileInputStream       internal_named_file_input_stream;
        NamedDataInputStream       internal_named_data_input_stream;
        NamedBufferedInputStream   internal_named_buffered_input_stream;
        NamedObjectInputStream     internal_named_object_input_stream;
        NamedSequenceInputStream   internal_named_sequence_input_stream;
        ByteArrayInputStream       internal_byte_array_input_stream;
        NamedFilterInputStream     internal_named_filter_input_stream;
        NamedPushbackInputStream   internal_named_pushback_input_stream;
        NamedLineNumberInputStream internal_named_line_number_input_stream;
        StringBufferInputStream    internal_string_buffer_input_stream;
        String                     file_name;
        ObjectServerException      object_server_exception;

        try {
            return_object = retry_object_input_stream.retryReadObject();
            oop = return_object;
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                    ObjectServerHeader.READ_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.READ_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerDebug.DEBUG_OBJECT_VALUE,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    oop.toString()));
            }
        } catch (Exception exception_object) {
            setFlag(false);
            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }

        if ((oop instanceof SODObject) == true) {
             
            object_type = parameter_type;
            parameter_index = parameter_index_number;

            if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && 
                    getLogStream() != null) {
                if (object_type == null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_TYPE_KEY,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ObjectServerDebug.DEBUG_NULL_KEY));
                } else {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_TYPE_KEY,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        object_type));
                }
            }

            if (getPersistentPath() != null) {
                file_name = getPersistentPath() + 
                    File.separator +
                    ObjectServerHeader.SUFFIX_SEPARATOR + 
                    object_type +
                    ObjectServerHeader.SUFFIX_SEPARATOR + 
                    Thread.currentThread().getName() +
                    ObjectServerHeader.SUFFIX_SEPARATOR +
                    String.valueOf(parameter_index) ;
            } else {
                file_name = ObjectServerHeader.SUFFIX_SEPARATOR + 
                    object_type +
                    ObjectServerHeader.SUFFIX_SEPARATOR + 
                    Thread.currentThread().getName() +
                    ObjectServerHeader.SUFFIX_SEPARATOR +
                    String.valueOf(parameter_index) ;
            }
            internal_byte_output_stream = null;
            internal_file_output_stream = null;
            if (ObjectServerHeader.isOutputStream(object_type) == true) {
        
                if (object_type.compareTo(ObjectServerHeader.JAVA_PIPEDOUTPUTSTREAM_TYPE_KEY) == 0 ) {
                    try {
                        internal_named_piped_input_stream
                                = new NamedPipedInputStream();
                    } catch (Exception exception_object) {
                        setFlag(false);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.NAMED_PIPED_INPUT_STREAM_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerError.NAMED_PIPED_INPUT_STREAM_EXCEPTION)));
                        throw object_server_exception;
                    }
                    try {
                        internal_named_piped_output_stream
                            = new NamedPipedOutputStream(internal_named_piped_input_stream);
                    } catch (Exception exception_object) {
                        setFlag(false);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.NAMED_PIPED_OUTPUT_STREAM_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerError.NAMED_PIPED_OUTPUT_STREAM_EXCEPTION)));
                        throw object_server_exception;
                    }
                    oop = internal_named_piped_output_stream;
                } else {
                    try {
                        internal_named_file_output_stream 
                            = new NamedFileOutputStream(file_name);
                    } catch (Exception exception_object) {
                        setFlag(false);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerError.NAMED_FILE_OUTPUT_STREAM_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerError.NAMED_FILE_OUTPUT_STREAM_EXCEPTION)));
                        throw object_server_exception;
                    }

                    if (object_type.compareTo(ObjectServerHeader.JAVA_OUTPUTSTREAM_TYPE_KEY) == 0 ||
                        object_type.compareTo(ObjectServerHeader.JAVA_FILEOUTPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_FILEOUTPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_FILEOUTPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_output_stream.toString()));
                        }
                        oop = internal_named_file_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_BYTEARRAYOUTPUTSTREAM_TYPE_KEY) == 0 ) {
                        try {
                            internal_byte_output_stream = new ByteArrayOutputStream();
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.JAVA_BYTE_ARRAY_OUTPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.JAVA_BYTE_ARRAY_OUTPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_byte_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_BUFFEREDOUTPUTSTREAM_TYPE_KEY
                        ) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_BUFFEREDOUTPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_BUFFEREDOUTPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_output_stream.toString()));
                        }
                        try {
                            internal_named_buffered_output_stream 
                                = new NamedBufferedOutputStream(internal_named_file_output_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_BUFFERED_OUTPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_BUFFERED_OUTPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_buffered_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_FILTEROUTPUTSTREAM_TYPE_KEY
                        ) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_FILTEROUTPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_FILTEROUTPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_output_stream.toString()));
                        }
                        try {
                            internal_named_filter_output_stream 
                                = new NamedFilterOutputStream(internal_named_file_output_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_FILTER_OUTPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_FILTER_OUTPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_filter_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_DATAOUTPUTSTREAM_TYPE_KEY
                        ) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_DATAINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_DATAINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_output_stream.toString()));
                        }
                        try {
                            internal_named_data_output_stream 
                                = new NamedDataOutputStream(internal_named_file_output_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_DATA_OUTPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_DATA_OUTPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_data_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY
                        ) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_OBJECTOUTPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_OBJECTOUTPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_output_stream.toString()));
                        }
                        try {
                            internal_named_object_output_stream 
                                = new NamedObjectOutputStream(internal_named_file_output_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_OBJECT_OUTPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_OBJECT_OUTPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_object_output_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_PRINTSTREAM_TYPE_KEY) == 0) {
                        try {
                            internal_named_print_stream 
                                = new NamedPrintStream(internal_named_file_output_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_PRINT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_PRINT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_print_stream;
                    } else {
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION)));
                        setFlag(false);
                        throw object_server_exception;
                    }
                }
                while (true) {
                    try {
                        return_object = retry_object_input_stream.retryReadObject();

                    } catch (Exception exception_object) {
                        if ((exception_object instanceof EOFException) == true) {
                            break;
                        }
                        setFlag(false);
                        throw exception_object;
                    }
                    if ((return_object instanceof EODObject) == true) {
                        if (getDebug() > 
                            oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && 
                            getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerHeader.READ_PARAMETER,
                                    getMessageResource().getMessage(
                                    ObjectServerHeader.READ_PARAMETER),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    return_object.getClass().getName(),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY));
                        }
                        break;
                    }
                }

            } else if (object_type.compareTo(ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) == 0 ||
                object_type.compareTo(ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0 ) {
                try {
                    internal_byte_output_stream
                        = new ByteArrayOutputStream();
                } catch (Exception exception_object) {
                    setFlag(false);
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_BYTE_ARRAY_OUTPUT_STREAM_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerError.JAVA_BYTE_ARRAY_OUTPUT_STREAM_EXCEPTION)));
                    throw object_server_exception;
                }
            } else {
                try {
                    internal_file_output_stream = new FileOutputStream( file_name);
                } catch (Exception exception_object) {
                    setFlag(false);
                    object_server_exception = new ObjectServerException(
                        getMessageResource().getFormattedMessage(
                            ObjectServerError.JAVA_FILE_OUTPUT_STREAM_EXCEPTION,
                            getMessageResource().getMessage(
                                ObjectServerError.JAVA_FILE_OUTPUT_STREAM_EXCEPTION)));
                    throw object_server_exception;
                }
            }

            if (ObjectServerHeader.isOutputStream(object_type) != true) {
                while (true) {
                    try {

                        return_object = retry_object_input_stream.retryReadObject();
                    } catch (Exception exception_object) {
                        if ((exception_object instanceof EOFException) == true) {
                            break;
                        }
                        setFlag(false);
                        throw exception_object;
                    }
                    if (return_object == null) { break ; }
                    if ((return_object instanceof EODObject) == true) {
                        break;
                    }
                    if (return_object.getClass().getName().compareTo(
                        ARRAY_BYTE_CLASS_NAME) != 0) {
                        break;
                    }
                    array = (byte[])return_object;

                    if (object_type.compareTo(
                        ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) == 0 ||
                        object_type.compareTo(
                        ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0 ) {
                        internal_byte_output_stream.write(array, 0, array.length);
                    } else {
                        internal_file_output_stream.write(array, 0, array.length);
                    }
                }

                if (object_type.compareTo(
                    ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) != 0 &&
                    object_type.compareTo(
                    ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) != 0 ) {
                    try {
                        internal_file_output_stream.close();
                        if (object_type.compareTo(
                            ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0) {
                            file_descriptor = new File(file_name);
                            if (file_descriptor.length() == 0) {

                                object_server_exception = new ObjectServerException(
                                    getMessageResource().getFormattedMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION,
                                        getMessageResource().getMessage(
                                            ObjectServerException.ASSERTION_EXCEPTION)));
                                setFlag(false);
                                throw object_server_exception;    
                            } 
                        }
                        try {
                            internal_named_file_input_stream 
                                = new NamedFileInputStream(file_name);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_FILE_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_FILE_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                    } catch (Exception second_exception_object) {
                        setFlag(false);
                        throw second_exception_object;
                    }
                   
                    if (object_type.compareTo(
                        ObjectServerHeader.JAVA_FILEINPUTSTREAM_TYPE_KEY) == 0 ||
                        object_type.compareTo(
                        ObjectServerHeader.JAVA_INPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_FILEINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_FILEINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        oop = internal_named_file_input_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_BUFFEREDINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_BUFFEREDINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_buffered_input_stream
                                = new NamedBufferedInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_BUFFERED_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_BUFFERED_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_buffered_input_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_SEQUENCEINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_SEQUENCEINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        ByteArrayInputStream byte_array_input_stream;
                        try {
                            byte_array_input_stream = new ByteArrayInputStream(
                            ObjectServerHeader.EMPTY_MESSAGE.getBytes() );
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.JAVA_BYTE_ARRAY_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.JAVA_BYTE_ARRAY_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }

                        try {
                            internal_named_sequence_input_stream
                                = new NamedSequenceInputStream(
                                internal_named_file_input_stream,
                                   byte_array_input_stream );
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_SEQUENCE_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_SEQUENCE_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
 
                        oop = internal_named_sequence_input_stream;
                    } else if (object_type.compareTo(
                        ObjectServerHeader.JAVA_DATAINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_DATAINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_DATAINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_data_input_stream
                                = new NamedDataInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_DATA_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_DATA_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_data_input_stream;
                    } else if (object_type.compareTo(ObjectServerHeader.JAVA_FILTERINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_FILTERINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_FILTERINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_filter_input_stream
                                = new NamedFilterInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_FILTER_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_FILTER_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_filter_input_stream;
                    } else if (object_type.compareTo(ObjectServerHeader.JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_LINENUMBERINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_LINENUMBERINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_line_number_input_stream
                                = new NamedLineNumberInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_LINE_NUMBER_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_LINE_NUMBER_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_line_number_input_stream;
                    } else if (object_type.compareTo(ObjectServerHeader.JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_PUSHBACKINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_PUSHBACKINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_pushback_input_stream
                            = new NamedPushbackInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_PUSHBACK_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_PUSHBACK_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_pushback_input_stream;
                    } else if (object_type.compareTo(ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY) == 0) {
                        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && getLogStream() != null) {
                            getLogStream().printLogln(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerDebug.DEBUG_JAVA_OBJECTINPUTSTREAM,
                                    getMessageResource().getMessage(
                                        ObjectServerDebug.DEBUG_JAVA_OBJECTINPUTSTREAM),
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    object_type,
                                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                    internal_named_file_input_stream.toString()));
                        }
                        try {
                            internal_named_object_input_stream
                                = new NamedObjectInputStream(internal_named_file_input_stream);
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.NAMED_OBJECT_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.NAMED_OBJECT_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_named_object_input_stream;

                    } else {

                        setFlag(false);
                        object_server_exception = new ObjectServerException(
                            getMessageResource().getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                getMessageResource().getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION)));
                        throw object_server_exception;    
                    }
                } else {
                    try {
                        internal_byte_output_stream.close();
                    } catch (Exception second_exception_object) {
                        setFlag(false);
                        throw second_exception_object;
                    }
                    if (object_type.compareTo(ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY) == 0) {
                        try {
                            internal_byte_input_stream = new ByteArrayInputStream(
                            internal_byte_output_stream.toByteArray() );
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.JAVA_BYTE_ARRAY_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.JAVA_BYTE_ARRAY_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        oop = internal_byte_input_stream;
                    } else if ( object_type.compareTo(ObjectServerHeader.JAVA_STRINGBUFFERINPUTSTREAM_TYPE_KEY) == 0 ) {
                        try {
                            internal_string_buffer_input_stream = 
                                new StringBufferInputStream( new String(
                                    internal_byte_output_stream.toByteArray() ));
                        } catch (Exception exception_object) {
                            setFlag(false);
                            object_server_exception = new ObjectServerException(
                                getMessageResource().getFormattedMessage(
                                    ObjectServerError.JAVA_STRING_BUFFER_INPUT_STREAM_EXCEPTION,
                                    getMessageResource().getMessage(
                                        ObjectServerError.JAVA_STRING_BUFFER_INPUT_STREAM_EXCEPTION)));
                            throw object_server_exception;
                        }
                        
                        oop = internal_string_buffer_input_stream;
                    }
                }
            } 
        }
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
            && getLogStream() != null) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerHeader.READ_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.READ_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    ObjectServerDebug.DEBUG_OBJECT_NAME,
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    oop.toString()));
        }
        setFlag(true);
        if (getDebug() > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (getLogStream() != null) {
                getLogStream().printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.READ_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.READ_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.toString()));
            }
        }
        return ;
    }
}

/**
 * This class provides InputStream for native type of variable
 *
 */
public class ArgumentInputStream {
    private static int debug = 0;
    private static int RETRY_NUMBER = 3;
    private int parameter_index = 0;
    private static int parameter_size  = 0;

    public final static String PARAMETER_ERROR = "Parameter error";
    public final static String READ_ARGUMENT_ERROR
        = "Read Argument error";
    public final static String OBJECT_INPUT_STREAM_IS_NULL
        = "ObjectInputStream is null.";

    public final static String DEBUG_READ_COUNT = "Read Count";
    public final static String DEBUG_CALL_READARGUMENT = "Call readArgument";
    public final static String DEBUG_SOCKETTIMEOUTEXCEPTION
        = "SocketTimeoutException ";
    public final static String DEBUG_RETRY_COUNT = "RETRY COUNT ";
    public final static String DEBUG_SET_DEBUG = "set Debug : ";
    public final static String DEBUG_BLANK = " ";
    public final static String DEBUG_OSERV = "OServ ";
    public final static String DEBUG_ACCEPT_REQUEST
        = "Accept Request";
    public final static String DEBUG_NOT_CONNECTED = "Not connected";
    public final static String DEBUG_RETRY = "Retry ";

    public final static int DEBUG_TIMEOUT_VALUE = 3000;
    public final static String DEBUG_ISSOCK = "issock ";
    public final static String DEBUG_OSSOCK = "ossock ";
    public final static String DEBUG_RECIEVE_REQUEST = "Recieve Request ";

    public final static String DEBUG_ARGUMENTINPUTSTREAM_ERROR
        = "ArgumentInputStream construction error";

    public final static String DEBUG_CALL_ARGUMENTINPUTSTREAM
        = "Call ArgumentInputStream";
    public final static String DEBUG_METHOD_NAME = "Method name ";
    public final static String DEBUG_CREATE_INSTANCE = "Create Instance ";

    public final static String DEBUG_SERVER_CMD = "Server cmd ";
    public final static String DEBUG_TRY_TO_READ_CMD = "Try to read cmd ";
    public final static String DEBUG_INPUT_SERV_PORT = "Input Serv Port ";
    public final static String DEBUG_NEXT_ARGUMENT = "nextArgument()";

    public final static String DEBUG_VARIABLE_TYPE = "Variable Type";

    private static RetryObjectInputStream retry_object_input_stream = null;

    private static String persistent_path = null;

    private static LogPrintStream logStream = null;
    private static MessageResource message_resource = new MessageResource();

    private intReadOperator      intopr = null;
    private longReadOperator     longopr = null;
    private shortReadOperator    shortopr = null;
    private floatReadOperator    floatopr = null;
    private doubleReadOperator   doubleopr = null;
    private objectReadOperator   objectopr = null;
    private booleanReadOperator  booleanopr = null;
    private byteReadOperator     byteopr = null;
    private charReadOperator     charopr = null;

    /**
     * This is constructor with ObjectInputStream
     */
    public ArgumentInputStream(RetryObjectInputStream ois) throws IOException {
        retry_object_input_stream = ois;
    }

    /**
     * This method sets Stream for switching.
     *
     * @param      PrintStream 
     */
    public void setLogStream(LogPrintStream ps) {
        logStream = ps;
    }

    /**
     * This method sets debug level.
     *
     * @param      int - debug level  
     * @return     void
     */
    public void setDebug(int level) {
        debug = level;
        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    message_resource.getFormattedMessage(
                        ObjectServerDebug.DEBUG_DEBUG_VALUE,
                        message_resource.getMessage(
                            ObjectServerDebug.DEBUG_DEBUG_VALUE),
                        String.valueOf(debug)));
            }
        }
    }

    public int getDebug() {
        return debug;
    }

    public LogPrintStream getLogStream() {
        return logStream;
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setPersistentPath(String path) {
        persistent_path = path;
    }

    public String getPersistentPath() {
        return persistent_path;
    }


    /**
     * @param      int  -  number of retry
     * @return     void
     *
     * set maximum value of retry count for reading via socket.
     */
    public void setRetryCount(int count) {

        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_SET_RETRY_COUNT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_SET_RETRY_COUNT),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(RETRY_NUMBER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(count)));
            }
        }

        if (RETRY_NUMBER < count) {
            RETRY_NUMBER = count;
        }
    }

    /**
     * @return    int
     * @throw     Exception
     *
     * read number of variable in table.
     */
    public int readCount() throws Exception {
        Integer   lsize;
        int   retry;

        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_READ_COUNT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
             }
         }
         try {
             lsize = (Integer)retry_object_input_stream.retryReadObject();
         } catch (Exception exception_object) {
             throw exception_object;
         }
         if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
             if (logStream != null) {
                  logStream.printLogln(
                     getMessageResource().getFormattedMessage(
                         ObjectServerHeader.READ_PARAMETER,
                         getMessageResource().getMessage(
                             ObjectServerHeader.READ_PARAMETER),
                         ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                         String.valueOf(lsize.intValue())));
             }
         }
         return lsize.intValue();
    }

    public int getintArgument() throws Exception {
        int  iop;
        Exception   exception_object;
        intReadOperator   int_read_operator;

        if (intopr == null) {
            int_read_operator = new intReadOperator();
            exception_object = int_read_operator.getException();
            throw exception_object;
        }
        try {
            iop = intopr.getintValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return iop;
    }

    public long getlongArgument() throws Exception {
        long        lop;
        Exception   exception_object;
        longReadOperator   long_read_operator;

        if (longopr == null) {
            long_read_operator = new longReadOperator();
            exception_object = long_read_operator.getException();
            throw exception_object;
        }
        try {
            lop = longopr.getlongValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return lop;
    }

    public float getfloatArgument() throws Exception {
        float  fop;
        Exception   exception_object;
        floatReadOperator   float_read_operator;

        if (floatopr == null) {
            float_read_operator = new floatReadOperator();
            exception_object = float_read_operator.getException();
            throw exception_object;
        }
        try {
            fop = floatopr.getfloatValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return fop;
    }

    public double getdoubleArgument() throws Exception {
        double  dop;
        Exception   exception_object;
        doubleReadOperator   double_read_operator;

        if (doubleopr == null) {
            double_read_operator = new doubleReadOperator();
            exception_object = double_read_operator.getException();
            throw exception_object;
        }
        try {
            dop = doubleopr.getdoubleValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return dop;
    }

    public short getshortArgument() throws Exception {
        short  sop;
        Exception   exception_object;
        shortReadOperator   short_read_operator;

        if (shortopr == null) {
            short_read_operator = new shortReadOperator();
            exception_object = short_read_operator.getException();
            throw exception_object;
        }
        try {
            sop = shortopr.getshortValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return sop;
    }

    public char getcharArgument() throws Exception {
        char  cop;
        Exception   exception_object;
        charReadOperator   char_read_operator;

        if (charopr == null) {
            char_read_operator = new charReadOperator();
            exception_object = char_read_operator.getException();
            throw exception_object;
        }
        try {
            cop = charopr.getcharValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return cop;
    }

    public byte getbyteArgument() throws Exception {
        byte               byop;
        Exception          exception_object;
        byteReadOperator   byte_read_operator;

        if (byteopr == null) {
            byte_read_operator = new byteReadOperator();
            exception_object = byte_read_operator.getException();
            throw exception_object;
        }
        try {
            byop = byteopr.getbyteValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return byop;
    }

    public boolean getbooleanArgument() throws Exception {
        boolean                bop;
        Exception              exception_object;
        booleanReadOperator    boolean_read_operator;

        if (booleanopr == null) {
            boolean_read_operator = new booleanReadOperator();
            exception_object = boolean_read_operator.getException();
            throw exception_object;
        }
        try {
            bop = booleanopr.getbooleanValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return bop;
    }

    public Object getObjectArgument() throws Exception {
        Object  oop;
        Exception   exception_object;
        objectReadOperator   object_read_operator;
        baseReadOperator     base_read_operator;

        if (objectopr == null) {
            base_read_operator = new baseReadOperator();
            exception_object = base_read_operator.getException();
          
            throw exception_object;
        }
        try {
            oop = objectopr.getObjectValue();
        } catch (Exception real_exception_object) {
            throw real_exception_object;
        }
        return oop;
    }

    public void resetOperator() {
        if (intopr != null) {
            intopr.setFlag(false);
        }
        if (longopr != null) {
            longopr.setFlag(false);
        }
        if (floatopr != null) {
            floatopr.setFlag(false);
        }
        if (doubleopr != null) {
            doubleopr.setFlag(false);
        }
        if (byteopr != null) {
            byteopr.setFlag(false);
        }
        if (charopr != null) {
            charopr.setFlag(false);
        }
        if (shortopr != null) {
            shortopr.setFlag(false);
        }
        if (booleanopr != null) {
            booleanopr.setFlag(false);
        }
        if (objectopr != null) {
            objectopr.setFlag(false);
        }
    }

    /**
     * @param    String      - name of field type
     * @return   Object      - reading variable
     * @throw    Exception
     *
     * reading Argument from variable file
     */
    public void nextArgument(String ptypes_name) 
        throws Exception {

        int                  retry = 0;
        Integer              lsize;
        Object               rag;
        String               field_type;
        ioReadOperator       ioopr;
	
        if (ptypes_name == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.PARAMETER_ERROR_EXCEPTION )));
            throw (Exception)ose;
        }
        if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_NEXT_ARGUMENT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ptypes_name);
            }
        }
        resetOperator();
        if (ptypes_name.compareTo(int.class.getName()) == 0) {
            intopr = new intReadOperator();
            ioopr = (ioReadOperator)intopr;
        } else 
        if (ptypes_name.compareTo(long.class.getName()) == 0) {
            longopr = new longReadOperator();
            ioopr = (ioReadOperator)longopr;
        } else 
        if (ptypes_name.compareTo(float.class.getName()) == 0) {
            floatopr = new floatReadOperator();
            ioopr = (ioReadOperator)floatopr;
        } else
        if (ptypes_name.compareTo(double.class.getName()) == 0) {
            doubleopr = new doubleReadOperator();
            ioopr = (ioReadOperator)doubleopr;
        } else
        if (ptypes_name.compareTo(boolean.class.getName()) == 0) {
            booleanopr = new booleanReadOperator();
            ioopr = (ioReadOperator)booleanopr;
        } else 
        if (ptypes_name.compareTo(byte.class.getName()) == 0) {
            byteopr = new byteReadOperator();
            ioopr = (ioReadOperator)byteopr;
        } else 
        if (ptypes_name.compareTo(char.class.getName()) == 0) {
            charopr = new charReadOperator();
            ioopr = (ioReadOperator)charopr;
        } else 
        if (ptypes_name.compareTo(short.class.getName()) == 0) {
            shortopr = new shortReadOperator();
            ioopr = (ioReadOperator)shortopr;
        } else {
            if (debug > oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln( 
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_VARIABLE_TYPE, 
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_VARIABLE_TYPE), 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ptypes_name ));
                }
            }
            try {
                objectopr = new objectReadOperator();
            } catch (Exception exception_object) {
                throw exception_object;
            }
            ioopr = (ioReadOperator)objectopr;
            if (getPersistentPath() != null) {
                ioopr.setPersistentPath(getPersistentPath());
            }
        }
      
        /**
         * setup debug environment
         */
        ioopr.setDebugLogStream(debug, logStream);
        ioopr.setMessageResource(getMessageResource());
        if (ObjectServerHeader.isInputStream(ptypes_name) == true 
            || ObjectServerHeader.isOutputStream(ptypes_name) == true) {
            if (objectopr != null) {
                objectopr.setParameterType(ptypes_name);
                objectopr.setParameterIndex(parameter_index);
            }
        }
        try {
            ioopr.readOperation(
                   (RetryObjectInputStream)retry_object_input_stream);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        parameter_index = parameter_index + 1;
        return ;
    }

    public static void main(String[] args) {
        int                 rt;
        byte[]              buffer;
        int                 int_value;
        long                long_value;
        float               float_value;
        double              double_value;
        short               short_value;
        char                char_value;
        byte                byte_value;
        boolean             boolean_value;
        Object              obj_rt;
        Object              check_object;
        MessageResource message_resource = new MessageResource();

        String              string_value;
        FileInputStream     file_input_stream;

        InputStream         parameter_input_stream_object;
        FileInputStream     parameter_file_input_stream;
        DataInputStream     parameter_data_input_stream;
        BufferedInputStream     parameter_buffered_input_stream;
        ByteArrayInputStream     parameter_byte_array_input_stream;
        ObjectInputStream     parameter_object_input_stream;
        LineNumberInputStream   parameter_line_number_input_stream;
        PushbackInputStream   parameter_pushback_input_stream;
        FilterInputStream   parameter_filter_input_stream;
        SequenceInputStream parameter_sequence_input_stream;

        OutputStream    parameter_output_stream_object;
        FileOutputStream    parameter_file_output_stream;
        DataOutputStream     parameter_data_output_stream;
        BufferedOutputStream     parameter_buffered_output_stream;
        ByteArrayOutputStream     parameter_byte_array_output_stream;
        ObjectOutputStream     parameter_object_output_stream;
        PrintStream   parameter_print_stream;
        FilterOutputStream   parameter_filter_output_stream;
        PipedOutputStream   parameter_piped_output_stream;

        ObjectInputStream   object_input_stream;
        RetryObjectInputStream retry_object_input_stream;
        ArgumentInputStream argument_input_stream;

        message_resource.setLanguage(ObjectServerHeader.OSERV_MESSAGE_LANGUAGE_VALUE);
        LogPrintStream    log_print_stream;
        try {
            log_print_stream = new LogPrintStream(System.out);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
       
        int  debug_level;
        try {
            debug_level = Integer.valueOf(
                System.getProperties().getProperty(
                ObjectServerHeader.OSERV_DEBUG_LEVEL_KEY)).intValue();
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        try {
            file_input_stream = new FileInputStream(args[0]);
            retry_object_input_stream = RetryObjectInputStream.valueOf(file_input_stream, message_resource, debug_level, log_print_stream);
            argument_input_stream 
                = new ArgumentInputStream(retry_object_input_stream);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        for (int i = 0; i < Argument.static_types.length; i++) {
            try {
                argument_input_stream.nextArgument(
                    Argument.static_types[i]
                );
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE1_TYPE) == 0) {
                    string_value = (String)argument_input_stream.getObjectArgument();
                    System.out.println(string_value);
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE2_TYPE) == 0) {
                    string_value = (String)argument_input_stream.getObjectArgument();
                    System.out.println(string_value);
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_INT_TYPE) == 0) {
                    int_value = argument_input_stream.getintArgument();
                    System.out.println(String.valueOf(int_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_LONG_TYPE) == 0) {
                    long_value = argument_input_stream.getlongArgument();
                    System.out.println(String.valueOf(long_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_FLOAT_TYPE) == 0) {
                    float_value = argument_input_stream.getfloatArgument();
                    System.out.println(String.valueOf(float_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_DOUBLE_TYPE) == 0) {
                    double_value = argument_input_stream.getdoubleArgument();
                    System.out.println(String.valueOf(double_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_SHORT_TYPE) == 0) {
                    short_value = argument_input_stream.getshortArgument();
                    System.out.println(String.valueOf(short_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BOOLEAN_TYPE) == 0) {
                    boolean_value = argument_input_stream.getbooleanArgument();
                    System.out.println(String.valueOf(boolean_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_CHAR_TYPE) == 0) {
                    char_value = argument_input_stream.getcharArgument();
                    System.out.println(String.valueOf(char_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BYTE_TYPE) == 0) {
                    byte_value = argument_input_stream.getbyteArgument();
                    System.out.println(String.valueOf(byte_value));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_INPUT_STREAM_OBJECT_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_input_stream_object = null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.InputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                    message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_INPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_input_stream_object = (InputStream)check_object;
                    System.out.println(parameter_input_stream_object.toString());

                    buffer = new byte[Argument.LINE1.length()];
                    rt = parameter_input_stream_object.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.LINE2.length()];
                    rt = parameter_input_stream_object.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_input_stream_object.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_FILE_INPUT_STREAM_TYPE) == 0) {

                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_file_input_stream = null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.FileInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                    message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_FILEINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_file_input_stream = (FileInputStream)check_object;
                    System.out.println(parameter_file_input_stream.toString());
                     
                    buffer = new byte[Argument.FILE_LINE1.length()];
                    rt = parameter_file_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.FILE_LINE2.length()];
                    rt = parameter_file_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_file_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_DATA_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_data_input_stream = (DataInputStream)null;
                        continue;
                    }
                    if ((check_object instanceof java.io.DataInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                    message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_DATAINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_data_input_stream = (DataInputStream)check_object;
                    System.out.println(parameter_data_input_stream.toString());
                    buffer = new byte[Argument.DATA_LINE1.length()];
                    rt = parameter_data_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.DATA_LINE2.length()];
                    rt = parameter_data_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_data_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BUFFERED_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_buffered_input_stream = (BufferedInputStream)null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.BufferedInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                                message_resource.getFormattedMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION,
                                    message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_BUFFEREDINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_buffered_input_stream = (BufferedInputStream)check_object;
                    System.out.println(parameter_buffered_input_stream.toString());
                    buffer = new byte[Argument.BUFFERED_LINE1.length()];
                    rt = parameter_buffered_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.BUFFERED_LINE2.length()];
                    rt = parameter_buffered_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_buffered_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BYTE_ARRAY_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_byte_array_input_stream = (ByteArrayInputStream)null;
                        continue;
                    }
                    if ((check_object instanceof java.io.ByteArrayInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_BYTEARRAYINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_byte_array_input_stream = (ByteArrayInputStream)check_object;
                    System.out.println(parameter_byte_array_input_stream.toString());
                    buffer = new byte[Argument.BYTEARRAY_LINE1.length()];
                    rt = parameter_byte_array_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.BYTEARRAY_LINE2.length()];
                    rt = parameter_byte_array_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_byte_array_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_OBJECT_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_object_input_stream = (ObjectInputStream)null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.ObjectInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_object_input_stream = (ObjectInputStream)check_object;
                    System.out.println(parameter_object_input_stream.toString());
                    obj_rt = parameter_object_input_stream.readObject();
                    System.out.println((String)obj_rt);
                    obj_rt = parameter_object_input_stream.readObject();
                    System.out.println((String)obj_rt);
                    parameter_object_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_LINE_NUMBER_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_line_number_input_stream = (LineNumberInputStream)null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.LineNumberInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_LINENUMBERINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_line_number_input_stream = (LineNumberInputStream)check_object;
                    System.out.println(parameter_line_number_input_stream.toString());
                    buffer = new byte[Argument.LINENUMBER_LINE1.length()];
                    rt = parameter_line_number_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.LINENUMBER_LINE2.length()];
                    rt = parameter_line_number_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_line_number_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_PUSHBACK_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_pushback_input_stream = (PushbackInputStream)null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.PushbackInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                        ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_PUSHBACKINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_pushback_input_stream = (PushbackInputStream)check_object;
                    System.out.println(parameter_pushback_input_stream.toString());

                    buffer = new byte[Argument.PUSHBACK_LINE1.length()];
                    rt = parameter_pushback_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.PUSHBACK_LINE2.length()];
                    rt = parameter_pushback_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    parameter_pushback_input_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_SEQUENCE_INPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_sequence_input_stream = (SequenceInputStream)null;
                        continue;
                    }
                    if ((check_object instanceof java.io.SequenceInputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_SEQUENCEINPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_sequence_input_stream = (SequenceInputStream)check_object;
                    System.out.println(parameter_sequence_input_stream.toString());
                    buffer = new byte[Argument.SEQUENCE_LINE1.length()];
                    rt = parameter_sequence_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    buffer = new byte[Argument.SEQUENCE_LINE2.length()];
                    rt = parameter_sequence_input_stream.read(buffer, 0, buffer.length);
                    System.out.println(new String(buffer, 0, buffer.length-1));
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_OUTPUT_STREAM_OBJECT_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_output_stream_object = (OutputStream)null;
                        continue;
                    }
                    if ((check_object instanceof java.io.OutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_OUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_output_stream_object = (OutputStream)check_object;
                    System.out.println(parameter_output_stream_object.toString());
                    buffer = Argument.LINE3.getBytes();
                    parameter_output_stream_object.write(buffer, 0, buffer.length);
                    buffer = Argument.LINE4.getBytes();
                    parameter_output_stream_object.write(buffer, 0, buffer.length);
                    parameter_output_stream_object.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_FILE_OUTPUT_STREAM_TYPE) == 0) {

                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_output_stream_object = (OutputStream)null;
                        continue;
                    }
                    if ((check_object instanceof java.io.FileOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_FILEOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_file_output_stream = (FileOutputStream)check_object;
                    System.out.println(parameter_file_output_stream.toString());

                    buffer = Argument.FILE_LINE1.getBytes();
                    parameter_file_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.FILE_LINE2.getBytes();
                    parameter_file_output_stream.write(buffer, 0, buffer.length);
                    parameter_file_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_DATA_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_data_output_stream = null;
                        continue;
                    }
                    if ((check_object instanceof java.io.DataOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_DATAOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_data_output_stream = (DataOutputStream)check_object;
                    System.out.println(parameter_data_output_stream.toString());

                    buffer = Argument.DATA_LINE1.getBytes();
                    parameter_data_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.DATA_LINE2.getBytes();
                    parameter_data_output_stream.write(buffer, 0, buffer.length);
                    parameter_data_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BUFFERED_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_buffered_output_stream = null;
                        continue;
                    }
                    if ((check_object instanceof java.io.BufferedOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_BUFFEREDOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_buffered_output_stream = (BufferedOutputStream)check_object;
                    System.out.println(parameter_buffered_output_stream.toString());

                    buffer = Argument.BUFFERED_LINE1.getBytes();
                    parameter_buffered_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.BUFFERED_LINE2.getBytes();
                    parameter_buffered_output_stream.write(buffer, 0, buffer.length);
                    parameter_buffered_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_BYTE_ARRAY_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_byte_array_output_stream = null;
                        continue;
                    }
                    if ((check_object instanceof ByteArrayOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_BYTEARRAYOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;

                    }
                    parameter_byte_array_output_stream = (ByteArrayOutputStream)check_object;
                    System.out.println(parameter_byte_array_output_stream.toString());
                    buffer = Argument.BYTEARRAY_LINE1.getBytes();
                    parameter_byte_array_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.BYTEARRAY_LINE2.getBytes();
                    parameter_byte_array_output_stream.write(buffer, 0, buffer.length);
                    parameter_byte_array_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_OBJECT_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = (ObjectOutputStream)argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_object_output_stream = null;
                        continue;
                    }
                    if ((check_object instanceof java.io.ObjectOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_OBJECTOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_object_output_stream = (ObjectOutputStream)check_object;
                    System.out.println(parameter_object_output_stream.toString());
                    parameter_object_output_stream.writeObject(
                        (Object)Argument.OBJECT_LINE1);
                    parameter_object_output_stream.writeObject(
                        (Object)Argument.OBJECT_LINE2);
                    parameter_object_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_PRINT_STREAM_TYPE) == 0) {
                    check_object = (PrintStream)argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_print_stream = null;
                        continue;
                    }
                    if ((check_object instanceof java.io.PrintStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_PRINTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_print_stream = (PrintStream)check_object;
                    System.out.println(parameter_print_stream.toString());
                    parameter_print_stream.print(Argument.PRINT_LINE1);
                    parameter_print_stream.print(Argument.PRINT_LINE2);
                    parameter_print_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_FILTER_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = (FilterOutputStream)argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_filter_output_stream = null;
                        continue;
                    } 
                    if ((check_object instanceof java.io.FilterOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_FILTEROUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_filter_output_stream = (FilterOutputStream)check_object;
                    System.out.println(parameter_filter_output_stream.toString());
                    buffer = Argument.FILTER_LINE1.getBytes();
                    parameter_filter_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.FILTER_LINE2.getBytes();
                    parameter_filter_output_stream.write(buffer, 0, buffer.length);
                    parameter_filter_output_stream.close();
                    continue;
                }
                if (Argument.static_types[i].compareTo(
                    Argument.VARIABLE_PIPED_OUTPUT_STREAM_TYPE) == 0) {
                    check_object = argument_input_stream.getObjectArgument();
                    if (check_object == null) {
                        parameter_piped_output_stream = null;
                        continue;
                    }
                    if ((check_object instanceof java.io.PipedOutputStream) != true) {
                        ObjectServerException object_server_exception;
                        object_server_exception = new ObjectServerException(
                            message_resource.getFormattedMessage(
                                ObjectServerException.ASSERTION_EXCEPTION,
                                message_resource.getMessage(
                                    ObjectServerException.ASSERTION_EXCEPTION),
                                ObjectServerHeader.BLANK_STRING,
                                ObjectServerHeader.JAVA_PIPEDOUTPUTSTREAM_TYPE_KEY,
                                ObjectServerHeader.BLANK_STRING,
                                check_object.getClass().getName()
                        ));
                        throw object_server_exception;
                    }
                    parameter_piped_output_stream = (PipedOutputStream)check_object;
                    System.out.println(parameter_piped_output_stream.toString());

                    buffer = Argument.FILTER_LINE1.getBytes();
                    parameter_piped_output_stream.write(buffer, 0, buffer.length);
                    buffer = Argument.FILTER_LINE2.getBytes();
                    parameter_piped_output_stream.write(buffer, 0, buffer.length);
                    parameter_piped_output_stream.close();
                    continue;
                }
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
            }
        }
    }
}
