/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ArgumentOutputStream.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2010/08/09
 *             2.1            - 2010/10/08
 *             2.1 2010/10/20 add setLogStream() and logStream variable
 *
 * This class extracts data basedon static variable type
 * and it writes to file for implementing persistent. 
 */

package oserv;

import java.lang.*;
import java.lang.reflect.*;
import java.net.*;
import java.io.*;

import oserv.*;

/**
 * Define an interface for writing operation
 */
interface ioWriteOperator {

    public void setDebugLogStream(int level, LogPrintStream ps) ;
    public void setData(Field fld, Object instance) throws Exception;
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception;
    public void setMessageResource(MessageResource msgrs) ;
    public MessageResource getMessageResource() ;
}

class baseWriteOperator  {
    boolean flag = false;
    protected static int debug = 0;
    protected static LogPrintStream logStream = null;
    private static MessageResource message_resource = new MessageResource();

    public void setDebugLogStream(int level, LogPrintStream ps) {
        if (level > debug) {
            debug = level;
        }
        if (ps != null) { 
            logStream = ps;
        }
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    protected ObjectServerException getObjectServerException(
        String  form_key,    String msg_key
    ) {
        ObjectServerException    object_server_exception;

        object_server_exception = new ObjectServerException(
            getMessageResource().getFormattedMessage(
                form_key,
                getMessageResource().getMessage( msg_key )));

        return object_server_exception;
    }
}

/**
 * For native int type, this calls getInt() and writeInt() method
 *
 * @see        ioWriteOperator
 */
class intWriteOperator extends baseWriteOperator implements ioWriteOperator {
    int int_value;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;

        try {
            int_value = fld.getInt(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_INT_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_INT_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        int.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(int_value)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;

        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    int.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(int_value)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(int.class.getName());
        network_object.setInt(int_value);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            int.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native long type, this call getLong() and writeLong() method
 *
 * @see        ioWriteOperator
 */
class longWriteOperator extends baseWriteOperator implements ioWriteOperator {
    long long_value;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;

        try {
            long_value = fld.getLong(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_LONG_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_LONG_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        long.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(long_value)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;

        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    long.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(long_value)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(long.class.getName());
        network_object.setLong(long_value);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            long.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native float type, this call getFloat() and writeFloat() method
 *
 * @see        ioWriteOperator
 */
class floatWriteOperator extends baseWriteOperator implements ioWriteOperator {
    float float_value;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;

        try {
            float_value = fld.getFloat(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_FLOAT_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_FLOAT_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        float.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(float_value)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;

        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    float.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(float_value)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(float.class.getName());
        network_object.setFloat(float_value);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            float.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native double type, this call getDouble() and writeDouble() method
 *
 * @see        ioWriteOperator
 */
class doubleWriteOperator extends baseWriteOperator implements ioWriteOperator {
    double double_value;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;
        try {
            double_value = fld.getDouble(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_DOUBLE_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_DOUBLE_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        double.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(double_value)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    double.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(double_value)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(double.class.getName());
        network_object.setDouble(double_value);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            double.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native boolean type, this call getBoolean() and writeBoolean() method
 *
 * @see        ioWriteOperator
 */
class booleanWriteOperator extends baseWriteOperator implements ioWriteOperator {
    boolean boolean_value;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;
        try {
            boolean_value = fld.getBoolean(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_BOOLEAN_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_BOOLEAN_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        boolean.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(boolean_value)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;

        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    boolean.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(boolean_value)
                ));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(boolean.class.getName());
        network_object.setBoolean(boolean_value);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            boolean.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native byte type, this call getByte() and writeByte() method
 *
 * @see        ioWriteOperator
 */
class byteWriteOperator extends baseWriteOperator implements ioWriteOperator {
    byte byop;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;
        try {
            byop = fld.getByte(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_BYTE_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_BYTE_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        byte.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(byop)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    byte.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(byop)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(byte.class.getName());
        network_object.setByte(byop);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            byte.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native char type, this call getChar() and writeChar() method
 *
 * @see        ioWriteOperator
 */
class charWriteOperator extends baseWriteOperator implements ioWriteOperator {
    char cop;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;
        try {
            cop = fld.getChar(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_CHAR_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_CHAR_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        char.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(cop)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    char.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(cop)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(char.class.getName());
        network_object.setChar(cop);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            char.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For native short type, this call getShort() and writeShort() method
 *
 * @see        ioWriteOperator
 */
class shortWriteOperator extends baseWriteOperator implements ioWriteOperator {
    short sop;

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;
        try {
            sop = fld.getShort(instance);
        } catch (Exception  exception_object) {
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_FIELD_GET_SHORT_EXCEPTION,
                ObjectServerError.JAVA_FIELD_GET_SHORT_EXCEPTION);
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        short.class.getName(),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(sop)));
            }
        }
    }
    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException   object_server_exception;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( getMessageResource().getFormattedMessage(
                    ObjectServerHeader.WRITE_PARAMETER,
                    getMessageResource().getMessage(
                        ObjectServerHeader.WRITE_PARAMETER),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    short.class.getName(),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    String.valueOf(sop)));
            }
        }
        NetworkObject  network_object;
        network_object = new NetworkObject();
        network_object.setName(short.class.getName());
        network_object.setShort(sop);
        try {
            object_output_stream.writeObject(network_object);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            short.class.getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
    }
}

/**
 * For basic Object type, this call get() and writeObject() method
 *
 * @see        ioWriteOperator
 */
class objectWriteOperator extends baseWriteOperator implements ioWriteOperator {
    Object oop;
    int             parameter_index = -1;
    EODObject       eod_object;
    NetworkObject   network_object;
    Field           local_field;
    String OBJECT_INPUT_STREAM_BIN = "bin";

    public void setData(Field fld, Object instance) throws Exception {
        ObjectServerException   object_server_exception;

        oop = fld.get(instance);

        if (oop != null && 
            oop.getClass().getName().compareTo(
                 ObjectServerHeader.JAVA_PIPEDINPUTSTREAM_TYPE_KEY) == 0 
        ) {

            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.NOT_SUPPORT_OBJECT_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.NOT_SUPPORT_OBJECT_EXCEPTION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.toString()));
            throw object_server_exception;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_READ_PARAMETER_FROM_INSTANCE),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.toString()));
            }
        }
    }

    public void setParameterIndex(int parameter_index_number) {
        parameter_index = parameter_index_number;
    }

    public void writeOperation(ObjectOutputStream object_output_stream) throws Exception {
        Object   save_object;
        SODObject sodobject_object;
        ObjectServerException   object_server_exception;
        byte[]   byte_value;
        
        save_object = null;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.WRITE_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.WRITE_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.toString()));
            }
        }
        try {
            if ( oop != null && (ObjectServerHeader.isOutputStream (oop.getClass().getName()) == true
              || ObjectServerHeader.isInputStream(oop.getClass().getName()) == true)) {
                save_object = oop;

                sodobject_object = new SODObject(SODObject.START_OF_DATA_MESSAGE);
                sodobject_object.setType(save_object.getClass().getName());
   
                sodobject_object.setParameterIndex(parameter_index);
                if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE && logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerHeader.WRITE_PARAMETER,
                            getMessageResource().getMessage(
                                ObjectServerHeader.WRITE_PARAMETER),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            save_object.getClass().getName(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            String.valueOf(parameter_index)
                    ));
                }
                oop = (Object)sodobject_object;
            } else {
                if (oop == null) {
                    byte_value = new byte[0];
                    oop = (Object)byte_value;
                } else {
                    ObjectOutputManager  object_output_manager;
                    object_output_manager = new ObjectOutputManager();
                    try {
                        byte_value = object_output_manager.toByteArray(oop);
                        oop = (Object)byte_value;
                    } catch (Exception exception_object) {
                        throw exception_object;
                    }
                }
            }
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            throw exception_object;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.WRITE_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.WRITE_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.toString()));
            }
        }
        local_field = null;
        if ( save_object != null && save_object.getClass().getName().compareTo(
                ObjectServerHeader.JAVA_OBJECTINPUTSTREAM_TYPE_KEY
            ) == 0) {
            try {
                local_field = save_object.getClass().getDeclaredField(OBJECT_INPUT_STREAM_BIN);
                save_object = (DataInputStream)local_field.get(save_object);
            } catch (Exception exception_object) {
                throw exception_object;
            }
        }

        try {
            object_output_stream.writeObject(oop);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }           
            }
            object_server_exception = getObjectServerException(
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
            throw object_server_exception;
        }
        if (save_object != null) {
            if (ObjectServerHeader.isInputStream(save_object.getClass().getName()) == true) {
                InputStream  input_stream;
                byte[]       buffer;
                byte[]       org_buffer;
                int          BUFFER_SIZE = 4096;
                int          rt;
                Object       rt_obj;
                input_stream = (InputStream)save_object;
                org_buffer = new byte[BUFFER_SIZE];

                while (true) {
                    try {
                        rt = input_stream.read(org_buffer, 0, BUFFER_SIZE);
                    } catch (Exception exception_object) {
                        if ((exception_object instanceof EOFException) == true) {
                            break;
                        }
                        object_server_exception = getObjectServerException(
                            ObjectServerError.JAVA_INPUT_STREAM_READ_IO_EXCEPTION,
                            ObjectServerError.JAVA_INPUT_STREAM_READ_IO_EXCEPTION);
                        throw object_server_exception;
                    } 
                    if (rt == -1) { break; }
                    if (org_buffer.length != rt) {
                        buffer = new byte[rt];
                        for (int i = 0; i < rt; i++) {
                            buffer[i] = org_buffer[i];
                        }
                    } else {
                        buffer = org_buffer;
                    }
                    try {
                        object_output_stream.writeObject(buffer);
                    } catch (Exception exception_object) {
                        object_server_exception = getObjectServerException(
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                        throw object_server_exception;
                    }
                }
            }
            eod_object = new EODObject(ObjectServerHeader.END_IOSTREAM);
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE
                && logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.WRITE_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.WRITE_PARAMETER),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        ObjectServerDebug.DEBUG_OBJECT_NAME,
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        eod_object.toString()));
            }
            try {
                object_output_stream.writeObject(eod_object);
            } catch (Exception exception_object) {
                object_server_exception = getObjectServerException(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION);
                throw object_server_exception;
            }
        }
    }
}

/**
 * This class provides OutputStream for native type of variable
 */
public class ArgumentOutputStream   {
    public final static String INTERNAL_ERROR = "Internal error";
    public final static String OBJECT_OUTPUT_STREAM_IS_NULL 
        = "ObjectOutputStream is null";
    public static int debug = 0;

    public static String temporary_directory = null;

    public static String DEBUG_SET_DEBUG = "set Debug : ";
    public static String DEBUG_FIELD_LENGTH = "Field length : ";
    public static String DEBUG_WRITEOBJECT = "writeObject() ";
    public static String DEBUG_END_WRITING_FIELD_LENGTH
        = "End writing field length";
    public static String DEBUG_FIELD_TYPE = " Field Type ";
    public static String DEBUG_SETDATA = "setData() ";
    public static String DEBUG_WRITEOPERATION = "writeOperation()";
    public final static String DEBUG_WRITE_ARGUMENT = "writeArgument()";

    public static LogPrintStream logStream = null;

    public ObjectOutputStream object_output_stream = null;
    private static MessageResource message_resource = new MessageResource();

    public ArgumentOutputStream(ObjectOutputStream oos) 
        throws IOException {
        object_output_stream = oos;
    }

    /**
     * This method sets Stream for switching.
     *
     * @param      int
     * @param      LogPrintStream 
     */
    public void setDebugLogStream(int level, LogPrintStream ps) {
        if (level > debug) {
            debug = level;
        }
        if (ps != null) { 
            logStream = ps;
        }
    }

    /**
     * This method sets debug level.
     *
     * @param      int - debug level  
     * @return     void
     */
    public void setDebug(int level) {
        debug = level;
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_DEBUG_VALUE,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_DEBUG_VALUE),
                        String.valueOf(debug)));
            }
        }
    }

    public void setMessageResource(MessageResource msgrs) {
        if (msgrs != null) {
            message_resource = msgrs;
        }
    }

    public MessageResource getMessageResource() {
        if (message_resource == null) {
            message_resource = new MessageResource();
        }
        return message_resource;
    }

    public void setTemporaryDirectory(String directory) {
        temporary_directory = directory;
    }

    public String getTemporaryDirectory() {
        return temporary_directory;
    }

    /**
     * This method write number of variables.
     *
     * @param      int - number of variable
     * @return     void
     * @throws     Exception
     */
    public void writeCount(Integer field_length) throws Exception {
        if (field_length == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.PARAMETER_ERROR_EXCEPTION )));
            throw (Exception)ose;
        }

        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln(
                    getMessageResource().getFormattedMessage(
                        ObjectServerHeader.WRITE_PARAMETER,
                        getMessageResource().getMessage(
                            ObjectServerHeader.WRITE_PARAMETER),
	                        String.valueOf((int)field_length)));
            }
        }
        try {
            object_output_stream.writeObject(field_length);
        } catch (Exception exception_object) {
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_WRITEOBJECT, 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }
            }
            throw exception_object;
        }
    }

    /**
     * This method writes parameter values with parameter type.
     *
     * @param  Field[]  - variable structure
     *         Object   - instance
     * @return void
     * @throws Exception
     */
    public void writeArgument(Field[] fld, Object instance) 
        throws Exception {

        String  field_type;
        intWriteOperator     intopr;
        longWriteOperator    longopr;
        floatWriteOperator   floatopr;
        doubleWriteOperator  doubleopr;
        booleanWriteOperator booleanopr;
        byteWriteOperator    byteopr;
        charWriteOperator    charopr;
        shortWriteOperator   shortopr;
        objectWriteOperator  objectopr;
        ioWriteOperator      ioopr;
        Object               oop;

        if (fld == null || instance == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerException.PARAMETER_ERROR_EXCEPTION,
                    getMessageResource().getMessage(
                        ObjectServerException.PARAMETER_ERROR_EXCEPTION )));
            throw (Exception)ose;
        }
        if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
            if (logStream != null) {
                logStream.printLogln( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_EXECUTE_KEY),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        DEBUG_WRITE_ARGUMENT,
                        getMessageResource().getMessage(
                            ObjectServerDebug.DEBUG_CODE_001)));
            }
        }

        for (int i = 0; i < fld.length; i++) {
            field_type = fld[i].getType().getName();
            if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                if (logStream != null) {
                    logStream.printLogln(
                        String.valueOf(i),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_FIELD_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_FIELD_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            field_type));
                }
            }
            objectopr = null;
            if (fld[i].getType().getName().compareTo(int.class.getName()) == 0) {

                intopr = new intWriteOperator();
                ioopr = (ioWriteOperator)intopr;
            } else
            if (fld[i].getType().getName().compareTo(long.class.getName()) == 0) {
                longopr = new longWriteOperator();
                ioopr = (ioWriteOperator)longopr;
            } else
            if (fld[i].getType().getName().compareTo(float.class.getName()) == 0) {
                floatopr = new floatWriteOperator();
                ioopr = (ioWriteOperator)floatopr;

            } else
            if (fld[i].getType().getName().compareTo(double.class.getName()) == 0) {
                doubleopr = new doubleWriteOperator();
                ioopr = (ioWriteOperator)doubleopr;
            } else
            if (fld[i].getType().getName().compareTo(boolean.class.getName()) == 0) {
                booleanopr = new booleanWriteOperator();
                ioopr = (ioWriteOperator)booleanopr;
            } else
            if (fld[i].getType().getName().compareTo(byte.class.getName()) == 0) {
                byteopr = new byteWriteOperator();
                ioopr = (ioWriteOperator)byteopr;
            } else
            if (fld[i].getType().getName().compareTo(char.class.getName()) == 0) {
                charopr = new charWriteOperator();
                ioopr = (ioWriteOperator)charopr;
            } else 
            if (fld[i].getType().getName().compareTo(short.class.getName()) == 0) {
                shortopr = new shortWriteOperator();
                ioopr = (ioWriteOperator)shortopr;
            } else {
                oop = fld[i].get(instance);
                if (oop != null && (oop instanceof Serializable) != true
 && ObjectServerHeader.isInputStream(fld[i].getType().getName()) != true 
 && ObjectServerHeader.isOutputStream(fld[i].getType().getName()) != true
                    && ObjectServerHeader.isObjectVariable(
                    oop.getClass().getName()) != true) {

                    ObjectServerException object_server_exception;
                    object_server_exception =
                        new ObjectServerException (
                        getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_IS_NOT_MOVABLE_EXCEPTION,
                            getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_IS_NOT_MOVABLE_EXCEPTION),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_MESSAGE_HEAD),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        String.valueOf(i),
                        ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                        oop.getClass().getName()  ));
                    throw object_server_exception;
                }
                objectopr = new objectWriteOperator();
                ioopr = (ioWriteOperator)objectopr;
            }
            ioopr.setDebugLogStream(debug, logStream);
            ioopr.setMessageResource(getMessageResource());
            try {
                ioopr.setData(fld[i], instance);
            } catch (Exception exception_object) {
                if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_SETDATA, 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }
                throw exception_object;
            }
            if (ObjectServerHeader.isInputStream(fld[i].getType().getName()) == true 
                || ObjectServerHeader.isOutputStream(fld[i].getType().getName()) == true) {
                if (objectopr != null) {
                    objectopr.setParameterIndex(i);
                }
            }
            try {
                ioopr.writeOperation( object_output_stream );
            } catch (Exception exception_object) {
                if (debug >= oserv.ObjectServerHeader.OSERV_DEBUG_FULL_VALUE) {
                    logStream.printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_EXCEPTION_KEY,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_EXCEPTION_KEY),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_WRITEOPERATION, 
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            exception_object.toString()));
                }
                throw exception_object;
            }
        }
    }

    private static String PARAMETER_ERROR = "Parameter error";

    private static String VAR1 = "ArgumentOutputStream test 1";
    private static String VAR2 = "ArgumentOutputStream test 2";

    private static String DEBUG_HEAD = "Debug";

    public static void main(String[] args) {
        FileOutputStream         file_output_stream;
        ObjectOutputStream       object_output_stream;
        ArgumentOutputStream     argument_output_stream;
        Argument                 argument;
        Field[]                  fields;
        Field[]                  params;
        String                   field_string;

        argument = new Argument();
        argument.method1(VAR1, VAR2);
        fields = argument.getClass().getFields();
        params = new Field[Argument.static_fields.length];

        for (int i = 0; i < Argument.static_fields.length; i++) {
            for (int j = 0; j < fields.length; j++) {
                field_string = fields[j].getName();
                if (field_string.compareTo(Argument.static_fields[i]) == 0) {
                    params[i] = fields[j];
                    break;
                }
            }
        }

        if (args.length != 1) {
            System.err.println(PARAMETER_ERROR);
            return;
        }
        try {
            file_output_stream = new FileOutputStream(args[0]);
            object_output_stream = new ObjectOutputStream(file_output_stream);
            argument_output_stream = new ArgumentOutputStream(object_output_stream);
            argument_output_stream.setDebugLogStream(
                ObjectServerHeader.OSERV_DEBUG_FULL_VALUE,
                new LogPrintStream(System.out));
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
        try {
            argument_output_stream.writeArgument(params, argument);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }
    }
}
