/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectService.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/10/03 for getting a service jar file or a file in the jar.
 *
 * This command gets service archive file or interface file in service archive
 * file via ObjectServer from remote ObjectServer.
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class ObjectService extends ObjectCommand {
    public static String OBJECT_SERVICE_ARGS = "f:h:";
    public static String FORWARD_HOST_OPTION  = "-h";
    public static String FORWARD_FILE_OPTION  = "-f";
    public static String COMMA               = ",";
    public static String EMPTY_STRING        = "";

    public static String DEBUG_SERVER        = "DEBUG Server";

    public static String PARAMETER_ERROR_EXCEPTION 
        = "Parameter error exception.";

    public static String TRACE_CONNECTSERVER   = "connecServer";
    public static String TRACE_CREATEFILE      = "createFile";
    public static String TRACE_GETFILE         = "createFile";
    public static String TRACE_SETFORWARD      = "setForward";
    public static String TRACE_GETSERVER       = "getServer";
    public static String TRACE_GETSERVICE      = "getService";

    private static URI  forward_host = null;
    private static int BUFFER_SIZE = 4096;

    private static int debug = 0;
    private static LogPrintStream logStream = null;

    public ObjectService() {
        if (getLogStream() == null) {
            setLogStream(new LogPrintStream(System.out));
        }
        String value_string;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }
    }

    public void setForward(String forward) throws Exception {
    
        try {
            forward_host = new URI(forward);
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_SETFORWARD, 0, exception_object);
            forward_host = null;
            throw exception_object;
        }
    }

    public URI getServer(String service) throws Exception {
        URI                   forward_entry;
        InetAddress           inet_address;
        String                hostname;
        String                port_string;
        Exception             real_exception_object;

        if (service == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            throw object_server_exception;
        }
        getProperties();
        port_string = getPropertyValue(OSERV_REQUEST_PORT_KEY);
        if (port_string != null) {
            if (getDebug() > OSERV_DEBUG_TRACE_VALUE && getLogStream() != null) {
                if (getLogStream() != null) {
                    getLogStream().printLogln(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_REQUEST_PORT,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_REQUEST_PORT),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            port_string));
                }
            }
        }
        try {
            inet_address = InetAddress.getLocalHost();
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_GETSERVER, 0, exception_object);
            real_exception_object = new Exception(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_SERVICE,
                    getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_SERVICE)));
            throw real_exception_object;
        }
        hostname = inet_address.getHostName();
        try {
            forward_entry = new URI(OSERV_SCHEME + 
                LIBRARY_SEPARATOR +
                OBJECT_SEPARATOR +
                OBJECT_SEPARATOR +
                hostname + 
                LIBRARY_SEPARATOR +
                port_string +
                OBJECT_SEPARATOR);
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_GETSERVER, 0, exception_object);
            real_exception_object = new Exception(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_SERVICE,
                    getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_SERVICE)));
            throw real_exception_object;
        }
        return forward_entry;
    }

    public String getFilePath(String service, String file) throws Exception {
        int       idx;
        String    target;
        String    directory;
        String    filepath;

        if (service == null || file == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            throw object_server_exception;
        }
        directory = new String(EMPTY_STRING);
        target = new String(EMPTY_STRING);
        idx = service.lastIndexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            directory = service.substring(0, idx);
            directory = replace(directory, OBJECT_DOT_SEPARATOR,
                            File.separator);
            target = service.substring(idx+1);
        }
        if (file != null) {
            idx = file.lastIndexOf(File.separator);
            if (idx >= 0) {
                directory = file.substring(0, idx);
                target = file.substring(idx+1);
            } else {
                if (File.separator.compareTo(DIRECTORY_SEPARATOR) != 0) {
                    idx = file.lastIndexOf(DIRECTORY_SEPARATOR);
                    if (idx >= 0) {
                        directory = file.substring(0, idx);
                        target = file.substring(idx+1);
                    } else {
                        directory = new String(EMPTY_STRING);
                        target = file;
                    }
                } else {
                    target = file;
                }
            }
        }
        return directory + File.separator + target;
    }

    public void getService(String service) throws Exception {
        byte[]            file_contents;
        byte[]            buffer;
        URI               command;
        String            servers;
        String            filepath;
        int               server_index;
        URI               server_host;
        File              file_descriptor;
        Long              file_length;
        Exception         exception_object;
        FileOutputStream  file_output_stream;
        InputStream       input_stream;
        int               read_count;
        int               request_count;
        int               offset;
        int               idx;
        RetryObjectInputStream    retry_object_input_stream;

        if (service == null) {
            ObjectServerException  object_server_exception;
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            throw object_server_exception;
        }
        idx = service.indexOf(OBJECT_DOT_SEPARATOR);
        if (idx >= 0) {
            filepath = service.substring(idx+1) 
                + SUFFIX_SEPARATOR + JAR_SUFFIX;
        } else {
            filepath = service + SUFFIX_SEPARATOR + JAR_SUFFIX;
        }

        file_descriptor = new File(filepath);
        if (file_descriptor.exists() == true) {
            exception_object = new Exception( getMessageResource().getFormattedMessage(
                ObjectServerError.FILE_ALREADY_EXIST_ERROR_SERVICE,
                getMessageResource().getMessage(
                    ObjectServerError.FILE_ALREADY_EXIST_ERROR_SERVICE)));
            throw exception_object;
        }
        file_output_stream = new FileOutputStream(file_descriptor);
        if (forward_host != null) {
            server_host = forward_host;
        } else {
            server_host = getServer(service);
        }
        command = new URI( server_host.toString() + 
            service + OPTION_SEPARATOR + FORWARD_COMMAND_KEY +
            QUERY_PARAMETER_SEPARATOR + FORWARD_SERVICE_COMMAND );
        try {
            createFile(command, filepath);
            return ;
        } catch (Exception real_exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_GETSERVICE, 0, real_exception_object);
            throw real_exception_object;
        }
    }

    public void closeSocket(Socket socket_object) {
        if (socket_object == null) {
            return;
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownOutput();
            } catch (Exception exception_object) {
                printExceptionMessage(ObjectService.class.getName(), TRACE_CONNECTSERVER, 0, exception_object);
            }
        }
        if (socket_object.isConnected() == true) {
            try {
                socket_object.shutdownInput();
            } catch (Exception exception_object) {
            }
        }
        if (socket_object.isConnected() == false) {
            getLogStream().printLogln(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CONNECTION_END_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CONNECTION_END_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    socket_object.toString()));
        } 
    }

    public Socket getConnectedSocket(URI command) throws Exception {
        byte[]  file = new byte[0];
        String             host_string;
        int                iport;
        Socket             connection_socket;
        InetSocketAddress  inet_socket_address;
        RetryObjectInputStream    retry_object_input_stream;
        OutputStream       output_stream;
        InputStream        input_stream;
        ObjectOutputStream object_output_stream;
        ObjectServerException   object_server_exception;

        if (command == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            throw object_server_exception;
        }
        host_string = command.getHost();
        iport = command.getPort();

        try {
            connection_socket = connectServer(host_string, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CONNECTSERVER, 0, exception_object);
            throw exception_object;
        }

        try {
            output_stream = connection_socket.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_FORWARD_KEY);
            object_output_stream.writeObject(command);
            connection_socket.shutdownOutput();
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw object_server_exception;
        }
        return connection_socket;
    }

    private void createFile(URI command, String filename) throws Exception {
        byte[]                    buffer;
        byte[]                    read_buffer;
        int                       offset;
        int                       request_count;
        int                       read_count;
        Long                      file_length;
        InputStream               input_stream;
        RetryObjectInputStream    retry_object_input_stream;
        FileOutputStream          file_output_stream;
        Byte[]                    byte_object_array;
        Byte                      byte_object;
        VariableByte              variable_byte;
        String                    vstring;
        int                       buffer_size;
        int                       retry_count = 0;
        String                    variable_field_type;
        Socket                    socket_object;
        ObjectServerException     object_server_exception;

        vstring  = getPropertyValue(OSERV_BUFFER_MAXIMUM_KEY);
        if (vstring != null) {
            try {
                buffer_size = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                buffer_size = BUFFER_SIZE;
            }
        } else {
            buffer_size = BUFFER_SIZE;
        }
        vstring = getPropertyValue(OSERV_RETRY_COUNT_KEY);
        if (vstring != null) {
            try {
                retry_count = Integer.valueOf(vstring).intValue();
            } catch (Exception exception_object) {
                printExceptionMessage(ObjectService.class.getName(), TRACE_CREATEFILE, 0, exception_object);
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_INTEGER_VALUE_OF_NUMBER_FORMAT_EXCEPTION)));
                throw object_server_exception;
            }
            if (retry_count > getRetryCount()) {
                setRetryCount(retry_count);
            }
        }

        buffer = new byte[buffer_size];
        try {
            socket_object = getConnectedSocket(command);
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CREATEFILE, 0, exception_object);
            throw exception_object;
        }
        try {
            input_stream = socket_object.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_CREATEFILE, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
/**
        retry_object_input_stream 
                = new RetryObjectInputStream(input_stream);
**/
        try {
            retry_object_input_stream 
                = RetryObjectInputStream.valueOf(input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream()
            );
        } catch (Exception exception_object) {
             printExceptionMessage(ObjectService.class.getName(), TRACE_CREATEFILE, 0, exception_object);
            throw exception_object;
        }
        retry_object_input_stream.setRetryCount(getRetryCount());
        if (getDebug() > 0) {
            if (getLogStream() == null) {
                setLogStream(new LogPrintStream(System.out));
            }
            retry_object_input_stream.setDebugLogStream(getDebug(), getLogStream());
        }

        try {
            file_output_stream = new FileOutputStream(filename);
        } catch (Exception  exception_object) {
             printExceptionMessage(ObjectService.class.getName(), TRACE_CREATEFILE, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_FILE_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }

        while (true) {
            try {
                variable_byte = (VariableByte)
                   retry_object_input_stream.retryReadObject();
            } catch (Exception exception_object) {
                if ((exception_object instanceof EOFException) == true) {
                    break;
                }
                throw exception_object;
            }
            read_buffer = variable_byte.getBytes();
            try {
                file_output_stream.write(read_buffer, 0, read_buffer.length);
            } catch (Exception exception_object) {
                object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_FILE_OUTPUT_STREAM_WRITE_IO_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_FILE_OUTPUT_STREAM_WRITE_IO_EXCEPTION)));
                throw object_server_exception;
            }
        }
        file_output_stream.close();
        
        closeSocket(socket_object);
    }

    public void getFile(String service, String file) throws Exception {
        byte[]            file_contents = new byte[0];
        byte[]            buffer;
        String            filepath;
        URI               server_host;
        URI               command;
        File              file_descriptor;
        FileOutputStream  file_output_stream;
        Exception         exception_object;
        int               read_count;
        int               request_count;
        int               offset;
        int               idx;
        InputStream       input_stream;
        RetryObjectInputStream    retry_object_input_stream;
        MakeDirectories   mkdir_object = null;
        ObjectServerException  object_server_exception;

        if (service == null || file == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            throw object_server_exception;
        }
        filepath = getFilePath(service, file);
        file_descriptor = new File(filepath);
        if (file_descriptor.exists() == true) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.FILE_ALREADY_EXIST_ERROR_SERVICE,
                    getMessageResource().getMessage(
                        ObjectServerError.FILE_ALREADY_EXIST_ERROR_SERVICE)));
            throw object_server_exception;
        }
        idx = filepath.indexOf(File.separator);
        if (idx > 0) {
            String directory;
            directory = filepath.substring(0, idx);
            if (directory != null && directory.length() > 0) {
                try {
                    mkdir_object = MakeDirectories.valueOf(directory,
                        getMessageResource(),
                        getDebug(),
                        getLogStream() );
                } catch (Exception real_exception_object) {
                    throw real_exception_object;
                }
                mkdir_object.mkdirs();
            }
        }
        if (forward_host != null) {
            server_host = forward_host;
        } else {
            server_host = getServer(service);
        }
        command = new URI( server_host.toString() + service +
                    OPTION_SEPARATOR +
                    FORWARD_COMMAND_KEY + QUERY_PARAMETER_SEPARATOR +
                    FORWARD_SERVICE_COMMAND + QUERY_SEPARATOR +
                    FORWARD_FILE_KEY + QUERY_PARAMETER_SEPARATOR +
                    filepath);
        try {
            createFile(command, filepath);
        } catch (Exception real_exception_object) {
            printExceptionMessage(ObjectService.class.getName(), TRACE_GETFILE, 0, real_exception_object);
            if (mkdir_object != null) {
                mkdir_object.rmdirs();
            }
            throw real_exception_object;
        }
        return ;
    }

    public static void main(String[] args) {
        ObjectService    object_service;
        ParseOptions     parse_options;
        String           value;
        String           filename;
        String           service;
        
        object_service = new ObjectService();

        try {
            parse_options = ParseOptions.valueOf(
                args, ObjectService.OBJECT_SERVICE_ARGS,
                null, null,
                object_service.getMessageResource(),
                object_service.getDebug(),
                object_service.getLogStream() );
        } catch (Exception exception_object) {
            System.err.println(
                object_service.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SERVICE,
                    object_service.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SERVICE)));
            return;
        }
        value = parse_options.getOption(FORWARD_HOST_OPTION);
        if (value != null) {
            try {
                object_service.setForward(value);
            } catch (Exception exception_object) {
                System.err.println(
                    object_service.getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_SERVICE,
                        object_service.getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_SERVICE)));
                return;
            }
        }
        filename = parse_options.getOption(FORWARD_FILE_OPTION);
        if (filename != null) {
            for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
                try {
                    service = parse_options.getRealArgument(i); 
                } catch (Exception exception_object) {
                    continue;
                }
                try {
                    object_service.getFile(service, filename);
                } catch (Exception exception_object) {
                    continue;
                }
            }
            return;
        }
        for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
            try {
                service = parse_options.getRealArgument(i); 
            } catch (Exception exception_object) {
                continue;
            }
            try {
                object_service.getService(service);
            } catch (Exception exception_object) {
                continue;
            }
        }
    }
}
