/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectSystem.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2012/03/12
 *
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * ObjectSystem -s | -p [service_names]
 */
public class ObjectSystem extends ObjectCommand {
    private final static String COMMAND_NAME = "ObjectSystem";
    private final static String USAGE_ARGUMENT = "-s | -p [service names]";
    public static String OBJECT_SYSTEM_ARGS = "s|p";
    public final static String SERIALIZABLE_OPTION   = "-s";
    public final static String PARALLEL_OPTION = "-p";

    public final static String TRACE_CLASSNAME = "oserv.ObjectSystem";
    public final static String TRACE_CONNECTSERVER = "setServiceType";

    public ObjectSystem() {
        super();

        String  value_string;

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }
    }

    public void setServiceType(String command, String type) throws Exception {
        int                 iport;
        String              vstring;
        String              host_string;
        Socket              socket_server;
        OutputStream        output_stream;
        ObjectOutputStream  object_output_stream;
        InputStream         input_stream;
        RetryObjectInputStream retry_object_input_stream;
        Object              object_instance;
        ObjectServerException object_server_exception;

        if (command == null || type == null) {
            object_server_exception = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_SYSTEM,
                        getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_SYSTEM)));
            throw object_server_exception;
        }
        host_string = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (host_string == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.HOSTNAME_NOT_FOUND_ERROR_SYSTEM,
                    getMessageResource().getMessage(
                        ObjectServerError.HOSTNAME_NOT_FOUND_ERROR_SYSTEM)));
            throw object_server_exception;
        }
        vstring = getPropertyValue(OSERV_REQUEST_PORT_KEY);

        if (vstring == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PORT_NOT_FOUND_ERROR_SYSTEM,
                    getMessageResource().getMessage(
                        ObjectServerError.PORT_NOT_FOUND_ERROR_SYSTEM)));
            throw object_server_exception;
        }
        try {
            iport = Integer.valueOf(vstring).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_VALUE_ERROR_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.PORT_VALUE_ERROR_TOOL)));
            throw object_server_exception;
        }

        try {
            socket_server = connectServer(host_string, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            throw exception_object;
        }
        try {
            output_stream = socket_server.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }

        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_SYSTEM_KEY);
            object_output_stream.writeObject(OSERV_SYSTEM_SET_SERVICE_TYPE);
            object_output_stream.writeObject(command);
            object_output_stream.writeObject(type);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw object_server_exception;
        }
        object_instance = null;
        try {
            input_stream = socket_server.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        try {
            retry_object_input_stream = RetryObjectInputStream.valueOf(
                input_stream,
                getMessageResource(),
                getDebug(),
                getLogStream() );
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            throw exception_object;
        }
        try {
            retry_object_input_stream.setRetryCount(getRetryCount());
            if (getDebug() > 0) {
                if (getLogStream() != null) {
                    retry_object_input_stream.setDebugLogStream(getDebug(), getLogStream());
                }
            }
            object_instance = retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECTSERVER, 0, exception_object);
            throw exception_object;
        }
        if (object_instance instanceof java.lang.Exception) {
            throw ((Exception)object_instance);
        }
    }

    /**
     * This method prints a usage.
     */
    public static void printUsage() {
        System.err.println(COMMAND_NAME + MESSAGE_SEPARATOR + USAGE_ARGUMENT);
    }

    public static void main(String[] args) {
        ParseOptions    parse_options;
        ObjectSystem    object_system;
        String          service_type;
        String          command;
        String          message_string;

        object_system = new ObjectSystem();
        try {
            parse_options = ParseOptions.valueOf(
                args, ObjectSystem.OBJECT_SYSTEM_ARGS,
                null, null,
                object_system.getMessageResource(),
                object_system.getDebug(),
                object_system.getLogStream() );
        } catch (Exception exception_object) {
            System.err.println(
                object_system.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_SYSTEM,
                    object_system.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_SYSTEM)));
            ObjectSystem.printUsage();
            return;
        }
        if (parse_options.checkOption(SERIALIZABLE_OPTION) == true) {
            service_type = ThreadTable.SERIALIZABLE_SERVICE;
        } else {
            if (parse_options.checkOption(PARALLEL_OPTION) == true) {
                service_type = ThreadTable.PARALLEL_SERVICE;
            } else {
                System.err.println(
                    object_system.getMessageResource().getFormattedMessage(
                        ObjectServerError.PARAMETER_ERROR_SYSTEM,
                        object_system.getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_SYSTEM)));
                ObjectSystem.printUsage();
                return;
            }
        }

        for (int i = 0; i < parse_options.getRealArgumentCount(); i++) {
            try {
                command = parse_options.getRealArgument(i); 
            } catch (Exception exception_object) {
                continue;
            }
            try {
                object_system.setServiceType(command, service_type);
            } catch (Exception exception_object) {
                message_string = object_system.getMessageResource().getMessage(
                ObjectServerException.REFUSE_CONNECTION);

                if (exception_object.getMessage() != null &&
                    (exception_object.getMessage().compareTo(message_string) == 0
                    || exception_object.getMessage().compareTo(ObjectServerException.REFUSE_CONNECTION) == 0) ) {
                        System.err.println(message_string);
                        continue;
                }

                System.err.println(
                    object_system.getMessageResource().getFormattedMessage(
                        ObjectServerError.IO_ERROR_SYSTEM,
                        object_system.getMessageResource().getMessage(
                            ObjectServerError.IO_ERROR_SYSTEM)));
                return;
            }
        }
    }
}
