/**
 * Copyright (C) All rights reserved.
 *
 * @Module     ObjectTool.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.0 2010/07/20
 *             2.1 2010/10/08
 *             2.1 2010/11/04 add procedure for property error
 *             2.1 2011/09/27 add forward
 *
 * Jar file name is same as Class name.
 * This class requires class name in order to allow to specify 
 * package name. 
 * But this class doesn't require package prefix in file name.
 * And this class looks for jar file under package name directory
 * when jar file that has class name under current directory doesn't
 * exist. 
 */

package oserv;

import java.lang.*;
import java.net.*;
import java.io.*;
import java.lang.reflect.*;

import oserv.*;

/**
 * This class provides a connection feature to oserver
 *
 * @see        oserv.ObjectServerHeader
 */
public class ObjectTool extends ObjectCommand {
    public final static String REGISTER_COMMAND = "Register";
    public final static String LIST_COMMAND = "List";
    public final static String FORWARD_LIST_COMMAND = "Forward";
    public final static String REMOVE_COMMAND = "Remove";
    public final static String FILE_COMMAND = "File";
    /**
     * Following variables are error messages.
     */
    public final static String CHECK_JAR_ERROR1 = "Jar file name error";
    public final static String FORMAT_ERROR = "Jar file format error";
    public final static String PARAMETER_ERROR = "Parameter error";
    public final static String ILLEGAL_OPTION = "Parameter error : illegal option was specified.";
    public final static String SYSTEM_ERROR    = "System error";
    public final static String IO_ERROR        = "I/O error : ";
    public final static String SOCKET_IO_ERROR = "Socket I/O error : ";
    public final static String UNKNOWN_HOST_ERROR = "Unknown Host error";

    public final static String EMPTY_STRING = "";

    public final static String JAR_FILE_HEAD = "Jar file ";
    public final static String EMPTY_TAIL    = " is empty.";
    public final static String NULL_TAIL    = " is null.";
    /**
     * Following variables are usage lines.
     */
    public final static String USAGE_LINE   = "[[-d] <classname> | [-l | -L]]";
    public final static String USAGE_LINE2  = "-d <classname> : delete the entry";
    public final static String USAGE_LINE3  = "-l : list entries.";
    public final static String USAGE_LINE4  = "-L : forward service list entries.";

    /**
    public final static String ARGS = "lLd:";
    **/
    public final static String ARGS = "lL|d:";

    public final static String LIST_HEADER = "Entry";
    public final static String EMPTY_LIST  = "Entry is empty.";

    public final static int ARGUMENT_HOSTNAME_INDEX = 0;
    public final static int ARGUMENT_CLASSNAME_INDEX = 1;
    public final static int ARGUMENT_COUNT = 2;

    public final static String COMMAND_NAME  = "ObjectTool";
    public final static String COMMAND_SPACE = "          ";

    public final static String DEBUG_CLOSE = "close()";
    public final static String DEBUG_READ  = "read()";
    public final static String DEBUG_SHUTDOWN = "shutdownInput(), shutdownOutput()";
    public final static String DEBUG_CLASSINPUTSTREAM_VALUEOF
        = "ClassInputStream.valueOf()";
    public final static String DEBUG_REQUIREREMOVEMENT
        = "requireRemovement()";
    public final static String DEBUG_REQUIRELIST
        = "requireList()";
    public final static String DEBUG_REQUIREREGISTRATION
        = "requireRegistration()";
    public final static String DEBUG_RETRYOBJECTINPUTSTREAM_VALUEOF
        = "RetryObjectInputStream.valueOf()";

    public final static String DEBUG_INTERNAL_ERROR = "Internal error :";
    public final static String DEBUG_BLANK = " ";
    public final static String DEBUG_NULL = "NULL";
    public final static String DEBUG_ISA = "inet_socket_address";
    public final static String DEBUG_CONNECT = "connect() ";
    public final static String DEBUG_TCP_NO_DELAY_TRUE  = "TcpNoDelay true";
    public final static String DEBUG_TCP_NO_DELAY_FALSE = "TcpNoDelay false";
    public final static String DEBUG_INETSOCKETADDRESS
                                             = "InetSocketAddress() ";
    public final static String DEBUG_THROWS_EXCEPTION = " throws Exception; ";
    public final static String DEBUG_SOCKET_EXCEPTION = "Socket Exception ";
    public final static String DEBUG_GETOUTPUTSTREAM = "getOutputStream() ";
    public final static String DEBUG_OUTPUTSTREAM_CONNECT
        = "public OutputStream connect(String hostname)";
    public final static String DEBUG_GENERATE_INTEGER_EXCEPTION
        = "Generate Integer Exception ";
    public final static String DEBUG_NEW_OBJECTOUTPUTSTREAM
        = "new ObjectOutputStream() ";
    public final static String DEBUG_FILE_ACCESS_ERROR = "File access error";
    public final static String DEBUG_FILE_SIZE_ERROR = "File size error";

    public final static String DEBUG_ARGUMENT_NUMBER = "Argument number ";
    public final static String DEBUG_COMMAND_OPTION = "Command Option";
    public final static String DEBUG_LIST_SIZE = "List size ";
    public final static String DEBUG_LIST_CONTENTS = "List contents ";
    public final static String DEBUG_FILE_NAME = "File name ";
    public final static String DEBUG_FILE_PATH = "File path ";
    public final static String DEBUG_FILE_LENGTH = "File length ";
    public final static String DEBUG_DELETE_COMMAND = "Delete command";
    public final static String DEBUG_BLOCK_COUNT = "Block count ";

    public final static String DEBUG_CONNECT_SERVER = "connectServer()";

    public final static String DEBUG_URL = "URL ";
    public final static String DEBUG_URL_LOADER = "URL loader ";

    public final static String DEBUG_CLASS_FORNAME_EXCEPTION
        = "Class forName Exception  ";
    public final static String DEBUG_CLIENT_SIDE_OBJECT_TABLE
        = "Client Side Object Table";

    public final static String DEBUG_REQUEST_LIST
        = "RequestList()";
    public final static String DEBUG_REGISTER_COMMAND
        = "RegisterCommand()";

    public final static String TRACE_CLASSNAME = "oserv.ObjectTool";
    public final static String TRACE_CONNECT = "connect";
    public final static String TRACE_CLOSE   = "close";
    public final static String TRACE_WRITE   = "write";
    public final static String TRACE_READ    = "read";
    public final static String TRACE_MAIN = "main";
    public final static String TRACE_EXECUTE = "execute";

    public final static String TRACE_REQUIREREGISTRATION
        = "requireRegistration";
    public final static String TRACE_REQUIREREMOVEMENT
        = "requireRemovement";
    public final static String TRACE_REQUIRELIST
        = "requireList";
    public final static String TRACE_GETLENGTH
        = "getLength";
    public final static String TRACE_CHECKPATH
        = "checkPath";
    public final static String TRACE_GETINPUTSTREAM
        = "getInputStream";
    public final static String TRACE_GETOUTPUTSTREAM
        = "getOutputStream";
    public final static String TRACE_SETSOTIMEOUT
        = "setSoTimeout";
    public final static String TRACE_CHECKJAR
        = "checkJar";
    public final static String TRACE_FLUSH
        = "flush";
    public final static String TRACE_EXEC = "exec";
    public final static String TRACE_SHUTDOWNINPUT
        = "shutdownInput";
    public final static String TRACE_SHUTDOWNOUTPUT
        = "shutdownOutput";
    public final static String TRACE_WRITECOMMAND
        = "writeCommand";

    /**
     * Following strings are options
     *
     * @see main()
     */
    public final static String DELETE_OPTION = "-d";
    public final static String HELP_OPTION   = "-h";
    public final static String LIST_OPTION   = "-l";
    public final static String FORWARD_LIST_OPTION   = "-L";

    public static int BUFFER_MAXIMUM = 1000;
    public static String workpath = null;
    public static int maximum = BUFFER_MAXIMUM;

    private static Socket              ssock;

    /**
     * This constructor is called for initializing property values.
     *
     *     OSERV_WORK_PATH_KEY
     *     OSERV_DEBUG_LEVEL_KEY
     *     OSERV_BUFFER_MAXIMUM_KEY
     *     OSERV_RETRY_COUNT_KEY
     *     OSERV_OBJECT_PORT_KEY
     */
    public ObjectTool() {
        super();
        int port_number;
        int retry_count = 0;

        String  value_string;

        if (getLogStream() == null) {
            setLogStream(new LogPrintStream(System.out));
        }

        getProperties();
        value_string = getPropertyValue(OSERV_MESSAGE_LANGUAGE_KEY);
        if (value_string != null) {
            setMessageLanguage(value_string);
        } else {
            setMessageLanguage(OSERV_MESSAGE_LANGUAGE_VALUE);
        }
        workpath = getPropertyValue(OSERV_WORK_PATH_KEY);
        if (workpath == null) {
            File pwd_file = new File(EMPTY_STRING);
            workpath = pwd_file.getAbsolutePath();
        }
        value_string = getPropertyValue(OSERV_DEBUG_LEVEL_KEY);
        if (value_string != null) {
            try {
                setDebug( Integer.valueOf(value_string).intValue() );
            } catch (Exception exception_object) {
                System.err.println( 
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_TOOL,
                        getMessageResource().getMessage(
                            ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_TOOL)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_TOOL,
                    getMessageResource().getMessage(
                        ObjectServerError.DEBUG_LEVEL_VALUE_ERROR_TOOL)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_BUFFER_MAXIMUM_KEY);
        if (value_string != null) {
            try {
                maximum = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_TOOL,
                        getMessageResource().getMessage (
                            ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_TOOL)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_TOOL,
                    getMessageResource().getMessage (
                        ObjectServerError.BUFFER_MAXIMUM_VALUE_ERROR_TOOL)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_RETRY_COUNT_KEY);
        if (value_string != null) {
            try {
                retry_count = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.RETRY_COUNT_VALUE_ERROR_TOOL,
                        getMessageResource().getMessage(
                            ObjectServerError.RETRY_COUNT_VALUE_ERROR_TOOL)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_TOOL,
                    getMessageResource().getMessage(
                        ObjectServerError.RETRY_COUNT_VALUE_NOT_FOUND_TOOL)));
            System.exit(ERROR_EXIT_CODE);
        }
        if (retry_count > getRetryCount()) {
            setRetryCount(retry_count);
        }
        value_string = getPropertyValue(
            ObjectServerHeader.OSERV_REQUEST_PORT_KEY);
        if (value_string != null) {
            try {
                port_number = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println(
                    getMessageResource().getFormattedMessage(
	                ObjectServerError.PORT_CONFIGURATION_ERROR_TOOL,
                        getMessageResource().getMessage(
	                    ObjectServerError.PORT_CONFIGURATION_ERROR_TOOL)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getMessageResource().getFormattedMessage(
                ObjectServerError.PORT_CONFIGURATION_NOT_FOUND_TOOL,
                getMessageResource().getMessage( 
                    ObjectServerError.PORT_CONFIGURATION_NOT_FOUND_TOOL)));
            System.exit(ERROR_EXIT_CODE);
        }
        value_string = getPropertyValue(OSERV_CONNECTION_TIMEOUT_KEY);
        if (value_string != null && value_string.length() != 0) {
            try {
                timeout = Integer.valueOf(value_string).intValue();
            } catch (Exception exception_object) {
                System.err.println( getMessageResource().getFormattedMessage(
                    ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER,
                    getMessageResource().getMessage( 
                        ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER)));
                System.exit(ERROR_EXIT_CODE);
            }
        } else {
            System.err.println( getMessageResource().getFormattedMessage(
                ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER,
                getMessageResource().getMessage( 
                    ObjectServerError.CONNECTION_TIMEOUT_VALUE_ERROR_SERVER)));
            System.exit(ERROR_EXIT_CODE);
        }
        if (getLogStream() == null) {
            setLogStream(new LogPrintStream(System.out));
        }
    }

    public int getTimeOut() {
        return timeout;
    }

    /**
     * This method call system method for putting a message to standard 
     * terminal.
     */
    public static void printUsage() {
        System.out.println(COMMAND_NAME + MESSAGE_SEPARATOR + USAGE_LINE);
        System.out.println(COMMAND_SPACE +MESSAGE_SEPARATOR + USAGE_LINE2);
        System.out.println(COMMAND_SPACE +MESSAGE_SEPARATOR + USAGE_LINE3);
        System.out.println(COMMAND_SPACE +MESSAGE_SEPARATOR + USAGE_LINE4);
    }

    /**
     * This method provides a feature to connect to server for registrating
     * Object.
     *
     * @param  String  -  Hostname
     * @throws Exception
     * @return OutputStream
     */
    public Socket connect(String hostname) throws Exception {
        int    iport;
        String sport;
        OutputStream      fot;
        InetSocketAddress inet_socket_address;

        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_OUTPUTSTREAM_CONNECT,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001)));
        if (hostname == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.HOSTNAME_PARAMETER_NULL_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.HOSTNAME_PARAMETER_NULL_TOOL)));
            throw ose;
        }

        sport = getPropertyValue(
            ObjectServerHeader.OSERV_REQUEST_PORT_KEY);
        if (sport == null) {
            ObjectServerException ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_CONFIGURATION_ERROR_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.PORT_CONFIGURATION_ERROR_TOOL)));
            throw ose;
        }
        try {
            iport = Integer.valueOf(sport).intValue();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECT, 0, exception_object);

            ObjectServerException ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PORT_VALUE_ERROR_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.PORT_VALUE_ERROR_TOOL)));
            throw ose;
        }

        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_CALL_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_CONNECT_SERVER,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_002)));
        try {
            ssock = connectServer(hostname, iport);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECT, 0, exception_object);
            throw exception_object;
        }
        if (ssock != null) {
            try {
                if (ssock.getTcpNoDelay() == true) {
                    printDebugTraceMessage(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ssock.toString(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_TCP_NO_DELAY_TRUE));
                } else {
                    printDebugTraceMessage(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            ssock.toString(),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            DEBUG_TCP_NO_DELAY_FALSE));
                }
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECT, 0, exception_object);
            }
        }
        try {
            ssock.setTcpNoDelay(true);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECT, 0, exception_object);
        }
        if (ssock != null) {
                try {
                    if (ssock.getTcpNoDelay() == true) {
                        printDebugTraceMessage(
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                ssock.toString(),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_TCP_NO_DELAY_TRUE));
                    } else {
                        printDebugTraceMessage(
                            getMessageResource().getFormattedMessage(
                                ObjectServerDebug.DEBUG_SOCKET_INFORMATION,
                                getMessageResource().getMessage(
                                    ObjectServerDebug.DEBUG_SOCKET_INFORMATION),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                ssock.toString(),
                                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                                DEBUG_TCP_NO_DELAY_FALSE));
                    }
                } catch (Exception exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_CONNECT, 0, exception_object);
                }
        }
        return ssock;
    }

    public void printIOConstructorError (Exception exception_object) {

        System.err.println(
            getMessageResource().getFormattedMessage(
                ObjectServerError.IO_CONSTRUCTION_ERROR_TOOL,
                getMessageResource().getMessage(
                    ObjectServerError.IO_CONSTRUCTION_ERROR_TOOL)));
 
        return;
    }

    /**
     * This method provides a feature to shutdown the socket line.
     *
     * @throws    Exception
     */
    public void close(Socket socket_object) throws Exception {
        InputStream            input_stream;
        Object                 return_object;
         
        if (socket_object == null) {
            ObjectServerException ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.CLOSE_PARAMETER_NULL_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.CLOSE_PARAMETER_NULL_TOOL)));
            throw ose;
        }
        if (socket_object.isConnected() == false) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CONNECTION_END_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CONNECTION_END_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    socket_object.toString()));
        } else {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_SHUTDOWN,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        }

        try {
            shutdownOutput(socket_object);
            shutdownInput(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_CLOSE, 0, exception_object);
            if (exception_object instanceof EOFException) {
                return;
            }
            throw exception_object;
        } 
        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_END_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_END_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TRACE_CLOSE));
    }

    /**
     * This method 
     *
     * @param    input file name
     * @param    buffer area
     * @return   void
     * @throws   Exception
     */
    public void read (String ifile, byte[] buffer) throws Exception {
        FileInputStream fin;
        DataInputStream din;
        ObjectServerException ose;

        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_READ,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001)));
       
        if (ifile == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.INPUT_FILE_NULL_READ_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.INPUT_FILE_NULL_READ_TOOL)));
            throw ose;
        }
        if (buffer == null) {
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.BUFFER_NULL_READ_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.BUFFER_NULL_READ_TOOL)));
            throw ose;
        }
        try {
            fin = new FileInputStream(ifile);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_READ, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION,
                    getMessageResource().getMessage( 
                    ObjectServerError.JAVA_FILE_INPUT_STREAM_FILE_NOT_FOUND_EXCEPTION)));
            throw ose;
        }

        din = new DataInputStream((InputStream)fin);
        try {
            din.read(buffer, 0, buffer.length);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_READ, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_IO_EXCEPTION,
                    getMessageResource().getMessage( 
                    ObjectServerError.JAVA_DATA_INPUT_STREAM_READ_IO_EXCEPTION)));
            throw ose;
        }    
        return;
    }


    public InputStream getInputStream(Socket socket_object) throws Exception {
        InputStream is;
        ObjectServerException    object_server_exception;

        if (socket_object == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw object_server_exception;
        }
        try {
            socket_object.setSoTimeout(timeout);
            is = socket_object.getInputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETINPUTSTREAM, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_INPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        return is;        
    }

    public OutputStream getOutputStream(Socket socket_object) throws Exception {
        OutputStream os;
        ObjectServerException    object_server_exception;

        if (socket_object == null) {
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
	            ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
	                ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw object_server_exception;
        }
        try {
            os = socket_object.getOutputStream();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_GETOUTPUTSTREAM, 0, exception_object);
            object_server_exception = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_SOCKET_GET_OUTPUT_STREAM_IO_EXCEPTION)));
            throw object_server_exception;
        }
        return os;
    }

    public void setSoTimeout(int tout) {
        if (tout < 0) {
            return;
        }
        try {
            ssock.setSoTimeout(tout);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_SETSOTIMEOUT, 0, exception_object);
        }
    }

    public void shutdownInput(Socket socket_object) {
        if (socket_object == null) {
            return;
        }
        if (socket_object.isConnected() == false) {
            return;
        }
        try {
            socket_object.shutdownInput();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_SHUTDOWNINPUT, 0, exception_object);
        }
    }

    public void shutdownOutput(Socket socket_object) {
        if (socket_object == null) {
            return;
        }
        if (socket_object.isConnected() == false) {
            return;
        }
        try {
            socket_object.shutdownOutput();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_SHUTDOWNOUTPUT, 0, exception_object);
        }
    }

    public void requireList(ObjectOutputStream object_output_stream) throws Exception {
        ObjectServerException    ose;
        RegisterCommand          list_command;

        String  list_request = new String(EMPTY_STRING);
        if (object_output_stream == null) {
            ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw (Exception)ose;
        }
        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TRACE_REQUIRELIST,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001)));

        try {
            list_command = RegisterCommand.listCommand(
                    getMessageResource(),
                    getDebug(),
                    getLogStream() );
            object_output_stream.writeObject(list_command);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIRELIST, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }    
    }

    public void requireRemovement(ObjectOutputStream object_output_stream, String jarclass) throws Exception {
        ObjectServerException ose;
        RegisterCommand          delete_command;

        if (object_output_stream == null || jarclass == null) {
            ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw (Exception)ose;
        }
        printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    TRACE_REQUIREREMOVEMENT,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        try {
            delete_command = RegisterCommand.deleteCommand(jarclass,
                    getMessageResource(),
                    getDebug(),
                    getLogStream() );
            object_output_stream.writeObject(delete_command);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIREREMOVEMENT, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }    
        return;
    }

    public void writeCommand(ObjectOutputStream object_output_stream)
        throws Exception {
        ObjectServerException ose;

        if (object_output_stream == null) {
            ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw ose;
        }
        try {
            object_output_stream.writeObject(OSERV_COMMAND_TYPE_OBJECT_KEY);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_WRITECOMMAND, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }
    }

    public void requireRegistration(
        ObjectOutputStream object_output_stream, 
        ClassInputStream class_input_stream) throws Exception {

        int    bkcount;
        int    length;
        int    rcount;
        byte[] buffer;
        String jarclass;
        DataOutputStream dot;
        ObjectServerException ose;
        RegisterCommand    register_command;

        if (object_output_stream == null || class_input_stream == null) {
            ose = new ObjectServerException( 
                getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage( 
                        ObjectServerError.PARAMETER_ERROR_TOOL)));
            throw ose;
        }
        printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    TRACE_REQUIREREGISTRATION,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        length = (int)class_input_stream.length();
        printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_REGISTER_COMMAND,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));

        try {
            register_command = RegisterCommand.registerCommand(
                class_input_stream.getClassPath(),
                class_input_stream.getType(),
                length,
                getMessageResource(),
                getDebug(),
                getLogStream() );
            object_output_stream.writeObject(register_command);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIREREGISTRATION, 0, exception_object);
            ose = new ObjectServerException(
                getMessageResource().getFormattedMessage(
                    ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION,
                    getMessageResource().getMessage(
                       ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_OBJECT_IO_EXCEPTION)));
            throw ose;
        }    

        SerializedByteArrayOutputStream bArray;
        bkcount = (length + maximum - 1)/maximum;
        for (int j = 0; j < bkcount; j++) {
            printDebugTraceMessage(
                    DEBUG_BLOCK_COUNT 
                    + String.valueOf(bkcount) + DEBUG_BLANK 
                    + String.valueOf(j));
            if (length > (j + 1) * maximum) {
                buffer = new byte[maximum];
            } else {
                buffer = new byte[length - (j * maximum)];
            }
            try {
                rcount = class_input_stream.read(buffer, 0, buffer.length);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIREREGISTRATION, 0, exception_object);
                ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_FILE_INPUT_STREAM_READ_IO_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_FILE_INPUT_STREAM_READ_IO_EXCEPTION)));
                throw ose;
            }
            if (rcount == -1) {
                break;
            }
            try {
                object_output_stream.write(buffer, 0, (int)rcount);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIREREGISTRATION, 0, exception_object);
                ose = new ObjectServerException(
                    getMessageResource().getFormattedMessage(
                        ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_IO_EXCEPTION,
                        getMessageResource().getMessage(
                            ObjectServerError.JAVA_OBJECT_OUTPUT_STREAM_WRITE_IO_EXCEPTION)));
                throw ose;
            }
        }
        try {
            object_output_stream.flush();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_REQUIREREGISTRATION, 0, exception_object);
        }
    }

    public void exec(String command, String class_name) 
        throws Exception {
        String                 message;
        String                 hostname;
        Object                 eos_object = new Object();
        Socket                 socket_object;
        OutputStream           output_stream;
        ObjectOutputStream     object_output_stream;
        InetAddress            inet_address;
        ClassInputStream       class_input_stream;
        RetryObjectInputStream retry_object_input_stream;

        printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_EXECUTE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    TRACE_EXEC,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_001)));
        if (command == null) {
            Exception              exception_object;
            message = getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_TOOL,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL));
            exception_object = new Exception(message);
            throw exception_object;
        }
        hostname = getPropertyValue(OSERV_SERVICE_HOSTNAME_KEY);
        if (hostname == null || hostname.length() == 0) {
            try {
                inet_address = InetAddress.getLocalHost();
            } catch (Exception exception_object) {
                printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
                Exception second_exception_object;
                message = getMessageResource().getFormattedMessage(
                    ObjectServerError.HOST_CONFIGURATION_ERROR_TOOL,
                    getMessageResource().getMessage(
                        ObjectServerError.HOST_CONFIGURATION_ERROR_TOOL));
                second_exception_object = new Exception(message);
                throw second_exception_object;
            }
            hostname = inet_address.getHostName();
        }
        class_input_stream = null;
        if (command.compareTo(REGISTER_COMMAND) == 0) {
            if (class_name == null) {
                Exception              exception_object;
                message = getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                exception_object = new Exception(message);
                throw exception_object;
            }
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_CLASSINPUTSTREAM_VALUEOF,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
            try {
                class_input_stream = ClassInputStream.valueOf(
                    class_name,
                    getMessageResource(),
                    getDebug(),
                    getLogStream() );

            } catch (Exception exception_object) {
                printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
                throw exception_object;
            }
        } else if (command.compareTo(REMOVE_COMMAND) == 0) {
            if (class_name == null) {
                Exception              exception_object;
                message = getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                exception_object = new Exception(message);
                throw exception_object;
            }
        }
        try {
            socket_object = connect(hostname);
        } catch (Exception exception_object) {
            printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
            throw exception_object;
        }
        try {
            output_stream = socket_object.getOutputStream();
            object_output_stream = new ObjectOutputStream(output_stream);
        } catch (Exception exception_object) {
            printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
            try {
                close(socket_object);
            } catch (Exception second_exception_object) {
                printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
            }
            throw exception_object;
        }
        retry_object_input_stream = null;
        if (command.compareTo(REGISTER_COMMAND) == 0) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_CLASSINPUTSTREAM_VALUEOF,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_REQUIREREGISTRATION,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
            try {
                writeCommand(object_output_stream);
                requireRegistration(object_output_stream, class_input_stream);
            } catch (Exception exception_object) {
                printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
                try {
                    close(socket_object);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
                }
                throw exception_object;
            }
        } else if (command.compareTo(REMOVE_COMMAND) == 0) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_REQUIREREMOVEMENT,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
            try {
                writeCommand(object_output_stream);
                requireRemovement(object_output_stream, class_name);
            } catch (Exception exception_object) {
                printExceptionMessage(
                    TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);

                try {
                    close(socket_object);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
                }
                throw exception_object;
            }
        } else if (command.compareTo(LIST_COMMAND) == 0 ||
            command.compareTo(FORWARD_LIST_COMMAND) == 0) {
            ClientSideObjectTable client_side_object_table ;
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_REQUIRELIST,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));

            try {
                writeCommand(object_output_stream);
                requireList(object_output_stream);
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
                try {
                    close(socket_object);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
                }
                throw exception_object;
            }

            try {
                retry_object_input_stream = RetryObjectInputStream.valueOf(
                        getInputStream(socket_object),
                        getMessageResource(),
                        getDebug(),
                        getLogStream() );
                retry_object_input_stream.setRetryCount(getRetryCount());
                client_side_object_table 
                    = new ClientSideObjectTable(retry_object_input_stream);
                client_side_object_table.setMessageResource(
                    getMessageResource());
            } catch (Exception exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
                try {
                    close(socket_object);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
                }
                throw exception_object;
            }
            if (command.compareTo(LIST_COMMAND) == 0) {
                client_side_object_table.printList();
            } else {
                client_side_object_table.printForwardList();
            }
        } else {
            Exception  exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_TOOL,
                getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL));
            exception_object = new Exception(message);
            throw exception_object;
        }

        if (retry_object_input_stream == null) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_CALL_KEY,
                    getMessageResource().getMessage( 
                        ObjectServerDebug.DEBUG_CALL_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    DEBUG_RETRYOBJECTINPUTSTREAM_VALUEOF,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_CODE_002)));
            try {
                retry_object_input_stream = RetryObjectInputStream.valueOf(
                    getInputStream(socket_object),
                    getMessageResource(),
                    getDebug(),
                    getLogStream() );
            } catch (Exception exception_object) {
                printIOConstructorError(exception_object);
                try {
                    close(socket_object);
                } catch (Exception second_exception_object) {
                    printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
                }
                throw exception_object;
            }
            retry_object_input_stream.setRetryCount(getRetryCount());
        }
        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_CALL_KEY,
                getMessageResource().getMessage( 
                    ObjectServerDebug.DEBUG_CALL_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                ObjectServerHeader.TRACE_RETRYREADOBJECT));
        try {
            eos_object = (EOSObject)retry_object_input_stream.retryReadObject();
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
            try {
                close(socket_object);
            } catch (Exception second_exception_object) {
                printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, second_exception_object);
            }
            throw exception_object;
        }
        if (eos_object instanceof EOSObject) {
            printDebugTraceMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_VALUE_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_VALUE_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    eos_object.toString()));
        }
        try {
            close(socket_object);
        } catch (Exception exception_object) {
            printExceptionMessage(TRACE_CLASSNAME, TRACE_EXECUTE, 0, exception_object);
            throw exception_object;
        }
        printDebugTraceMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_END_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_END_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TRACE_EXEC));
    }


    public static void main(String[] args) {
        int                idx;
        String             message;
        String             command;
        String             filename;
        String             class_name;
        ObjectTool         ofunc;
        ParseOptions       parse_options;

        ofunc = new ObjectTool();
        try {
            parse_options = ParseOptions.valueOf( 
                args, ObjectTool.ARGS, null, null,
                ofunc.getMessageResource(),
                ofunc.getDebug(),
                ofunc.getLogStream() );
        } catch (Exception exception_object) {
            message = ofunc.getMessageResource().getFormattedMessage(
                ObjectServerError.PARAMETER_ERROR_TOOL,
                ofunc.getMessageResource().getMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL));
            System.err.println(message);
            ObjectTool.printUsage();
            return;
        }
        if (parse_options.checkOption(HELP_OPTION) == true) {
            ObjectTool.printUsage();
            return;
        }
        class_name = null;
        if ( parse_options.checkOption(LIST_OPTION) == true ) {
            if (parse_options.getRealArgumentCount() > 0) {
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                System.err.println(message);
                ObjectTool.printUsage();
                return;
            }
            command = LIST_COMMAND;
        } else if ( parse_options.checkOption(FORWARD_LIST_OPTION) == true) {
            if (parse_options.getRealArgumentCount() > 0) {
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                System.err.println(message);
                ObjectTool.printUsage();
                return;
            }
            command = FORWARD_LIST_COMMAND;
        } else if (parse_options.checkOption(DELETE_OPTION) == true ) {
            class_name = parse_options.getOption(DELETE_OPTION);
            command = REMOVE_COMMAND;
        } else {
            if (parse_options.getRealArgumentCount() != 1) {
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                            ObjectServerError.PARAMETER_ERROR_TOOL));
                System.err.println(message);
                ObjectTool.printUsage();
                return;
            }
            try {
                class_name = parse_options.getRealArgument(0);  
            } catch (Exception exception_object) {
                ofunc.printExceptionMessage(TRACE_CLASSNAME, TRACE_MAIN, 0, exception_object);
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                System.err.println(message);
                ObjectTool.printUsage();
                return;
            }
            if (class_name == null) {
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.PARAMETER_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                        ObjectServerError.PARAMETER_ERROR_TOOL));
                System.err.println(message);
                ObjectTool.printUsage();
                return;
            }
            command = REGISTER_COMMAND;
        }
        try {
            ofunc.exec(command, class_name); 
        } catch (Exception exception_object) {
            message = exception_object.getMessage();
            if (message == null) {
                message = ofunc.getMessageResource().getFormattedMessage(
                    ObjectServerError.INTERNAL_ERROR_TOOL,
                    ofunc.getMessageResource().getMessage(
                        ObjectServerError.INTERNAL_ERROR_TOOL));
            }
            System.err.println( message );
            return;
        }
        ofunc.printDebugTraceMessage(
            ofunc.getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_END_KEY,
                ofunc.getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_END_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                TRACE_MAIN));
    }
}
