/**
 * Copyright (C) All rights reserved.
 *
 * @Module     URITable.java
 * @Author     Takahiko Tanaka
 *             uewxiun@pb3.so-net.ne.jp
 * @Version    2.1 2011/11/10
 */

package oserv;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * URITable
 */
public class URITable extends SerializableObject {

    private URI[] uri_table = new URI[0];
    private int   uri_count = 0;
    private int   uri_point = -1;

    public final static String ITEM_SEPARATOR = ",";

    public final static int NETWORK_BYTE = 4;

    public final static char CHAR_SMALL_A = 'a';
    public final static char CHAR_SMALL_F = 'f';
    public final static char CHAR_LARGE_A = 'A';
    public final static char CHAR_LARGE_F = 'F';
    public final static String HEX_HEAD = "0x";

/*
    private int debug = 0;
*/

/**
    public final static String OVERFLOW_EXCEPTION = "Overflow exception";
**/

    public final static String COMMAND_KEY = "command";
    public final static String NETWORK_COMMAND_KEY = "network";
    public final static String NETMASK_KEY = "netmask";

    public final static String DEBUG_TABLE_SIZE = "Table size";
    public final static String DEBUG_ITEM  = "Item";
    public final static String DEBUG_ITEM_COUNT  = "Item count";
    public final static String DEBUG_BLANK = " ";
    public final static String DEBUG_CALL_MAKE_URITABLE
        = "Call make URI table";
    public final static String DEBUG_IS_VALID = "isValid()";
    public final static String DEBUG_NULL = "NULL";
    public final static String DEBUG_HOST_ADDRESS = "Host address";
    public final static String DEBUG_ADDRESS = "Address";
/*
    public final static String DEBUG_EXCEPTION_KEY = "Exception";
*/
    public final static String DEBUG_NETMASK = "Netmask";

    public final static String DEBUG_LOAD_URI_TABLE = "loadURITable()";
    public final static String DEBUG_CURRENT_HOST_ADDRESS 
        = "Current host address";
    public final static String DEBUG_NETWORK_MASK 
        = "Network mask";
    public final static String DEBUG_TARGET_ADDRESS
        = "Target address";
    public final static String DEBUG_URI_TABLE_MATCH
        = "URI table match";
    public final static String DEBUG_NEXT_ENTRY
        = "next entry";
    public final static String DEBUG_HAS_MORE_ENTRIES
        = "hasMoreEntries";
    public final static String DEBUG_GET_HOSTADDRESS
        = "getHostAddress";
    public final static String DEBUG_MATCH_HOSTADDRESS
        = "match Host Address";

    private static ArrayList debug_box = new ArrayList();
/*
    private static MessageResource message_resource = new MessageResource();
*/

    public URITable() {
    }

/*
    public void setMessageResource(MessageResource msgr) {
        message_resource = msgr;
    }
*/

    public static ArrayList getDebugBox() {
        return debug_box;
    }

    public void clearDebugBox() {
        debug_box.clear();
    }

    public void setDebugBox(ArrayList alist) {
        debug_box.addAll(alist);
    }

    public static void addDebugMessage(String message) {
        debug_box.add(message);
    }

    public static URITable makeURITable(String entries) throws Exception {
        URITable uri_table_entry;
        uri_table_entry = new URITable();

        uri_table_entry.addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_CALL_MAKE_URITABLE,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001)) +
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY +
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_ENTRY_NAME,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_ENTRY_NAME),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                entries));
        try {
            uri_table_entry.loadURITable(entries);
        } catch (Exception exception_object) {
            throw exception_object;
        }
        return uri_table_entry;      
    }

    public int countEntries() {
        return uri_count;
    }

    public boolean hasMoreEntries() {
 
        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_HAS_MORE_ENTRIES,
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001))+
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY+
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_COUNT_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_COUNT_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(uri_count))+
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY+
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_POINT_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_POINT_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(uri_point) ));

        if (uri_count > 0 && (uri_point != -1 && uri_point >= uri_count)) {
            return false;
        }
        return true;
    }

    public boolean isNetwork(URI uri_entry) {
        String query_string;
        String netmask;

        if (uri_entry == null) {
            return false;
        }
        query_string = uri_entry.getQuery();
        try {
            netmask = oserv.ObjectServerHeader.getQueryParameter(
                            query_string, NETMASK_KEY);
        } catch (Exception exception_object) {
            return false;
        }
        if (netmask == null) {
            return false;
        }
        return true;
    }

    private long convAddressToLong(String network) {
        long host_value;

        if (network == null) {
            return 0;
        }
        StringTokenizer stk = new StringTokenizer(network, 
                          oserv.ObjectServerHeader.ADDRESS_SEPARATOR);

        if (stk.countTokens() != NETWORK_BYTE) {
            return 0;
        }
        host_value = 0;
        while (stk.hasMoreTokens()) {
            host_value = (host_value << 8) 
                        + Long.decode(stk.nextToken()).longValue();
        }
        return host_value;
    }

    public boolean matchNetwork(URI uri_entry, String address) {
        String    query_string;
        String    command;
        String    netmask;
        String    host_address;
        long      host_value;
        long      target_value;
        long      netmask_value;
        boolean   hex_flag;

        clearDebugBox();
        if (uri_entry == null && address == null) {
            return false;
        }
        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_TARGET_ADDRESS,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001))+
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY+
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_HOST_ADDRESS,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_HOST_ADDRESS),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                address));

        if (isNetwork(uri_entry) != true) {
            return false;
        }

        query_string = uri_entry.getQuery();
        host_address = getHostAddress(uri_entry);

        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_HOST_ADDRESS,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_HOST_ADDRESS),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY, 
                host_address));
        host_value = convAddressToLong(host_address) ;
        target_value = convAddressToLong(address);

        try {
            netmask = oserv.ObjectServerHeader.getQueryParameter(
                                    query_string, NETMASK_KEY);
        } catch (Exception exception_object) {
            return false;
        }
        hex_flag = false;
        if (netmask.indexOf(HEX_HEAD) != 0) {
            for (int i = 0; i < netmask.length(); i++) {
                if ( (netmask.charAt(i) >= CHAR_SMALL_A &&
                    netmask.charAt(i) <= CHAR_SMALL_F) ||
                    (netmask.charAt(i) >= CHAR_LARGE_A &&
                    netmask.charAt(i) >= CHAR_LARGE_F)  ){
                    hex_flag = true;
                }
            }
        }
        if (hex_flag == true) {
            netmask_value = Long.decode(HEX_HEAD + netmask).longValue();
        } else {
            netmask_value = Long.decode(netmask).longValue();
        }
        addDebugMessage(
            DEBUG_NETWORK_MASK+
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY+
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_VALUE_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_VALUE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                Integer.toHexString((int)netmask_value)));

        if ((host_value & netmask_value) ==
                    (target_value & netmask_value) ) {
            return true;
        }
        return false;
    }

    public String getHostAddress(URI uri_entry) {
        String  hostname;
        String  host_address;

        host_address = null;
        hostname = uri_entry.getHost();
        try {
            host_address = InetAddress.getByName(hostname).getHostAddress();
        } catch (Exception exception_object) {
            return host_address;
        }
        return host_address;
    }

    public boolean isValid(String address) {
        URI    uri_entry;
        String host_address;

        if (address == null) {
            return false;
        }

        reset() ;
        while (hasMoreEntries()) {
            try {
                uri_entry = nextEntry();
            } catch (Exception exception_object) {
                return false;
            }
            if (isNetwork(uri_entry) == true) {
                if (matchNetwork(uri_entry, address) == true) {
                    addDebugMessage(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_HOST_ADDRESS,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_HOST_ADDRESS),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            address));
                    return false;
                }
            } else {
                addDebugMessage(DEBUG_GET_HOSTADDRESS + DEBUG_BLANK
                    + uri_entry.toString());
                host_address = getHostAddress(uri_entry);
                if (address.compareTo(host_address) == 0) {
                    addDebugMessage(
                        getMessageResource().getFormattedMessage(
                            ObjectServerDebug.DEBUG_HOST_ADDRESS,
                            getMessageResource().getMessage(
                                ObjectServerDebug.DEBUG_HOST_ADDRESS),
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            host_address,
                            ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                            address));
                    return false;
                }
            }
        } 
        return true;
    }

    public URI nextEntry() throws Exception {
        URI  uri_entry;

        if (uri_point == -1) {
            uri_point = 0;
        }
        if (uri_count > 0 && uri_point > uri_count) {
            String    message;
            Exception exception_object;

            message = getMessageResource().getFormattedMessage(
                ObjectServerError.OVERFLOW_EXCEPTION_URITABLE,
                getMessageResource().getMessage(
                    ObjectServerError.OVERFLOW_EXCEPTION_URITABLE));
            exception_object = new Exception(message);
            throw exception_object;
        }
        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_EXECUTE_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_EXECUTE_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                DEBUG_NEXT_ENTRY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_CODE_001))+
            ObjectServerDebug.DEBUG_BLANK_STRING_KEY+
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_POINT_KEY,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_POINT_KEY),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(uri_point)));

        uri_entry = uri_table[uri_point];
        uri_point = uri_point + 1;
        return uri_entry;
    }

    public URI getEntry(int index) throws Exception {
        URI  uri_entry;
        uri_entry = uri_table[index];
        return uri_entry;
    }

    public void reset() {
        uri_point = 0;
    }

    public void loadURITable(String entries) throws Exception {
        int               count;
        String            entry;
        StringTokenizer   string_tokenizer;

        if (entries == null) {
            return;
        }
        string_tokenizer = new StringTokenizer(entries, ITEM_SEPARATOR);
        count = string_tokenizer.countTokens();
        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_ITEM_COUNT,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_ITEM_COUNT),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(count) ));
        uri_table = new URI[count];
        uri_count = 0;
        for (int i = 0; i < count; ) {
            entry = string_tokenizer.nextToken();
            if (entry == null || entry.length() == 0) {
                count--;
                continue;
            }
            addDebugMessage(
                getMessageResource().getFormattedMessage(
                    ObjectServerDebug.DEBUG_ITEM_KEY,
                    getMessageResource().getMessage(
                        ObjectServerDebug.DEBUG_ITEM_KEY),
                    ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                    entry));
            try {
                uri_table[i] = new URI(entry);
            } catch (Exception exception_object) {
                count--;
                continue;
            }
            uri_count++;
            i++;
        }
        addDebugMessage(
            getMessageResource().getFormattedMessage(
                ObjectServerDebug.DEBUG_TABLE_SIZE,
                getMessageResource().getMessage(
                    ObjectServerDebug.DEBUG_TABLE_SIZE),
                ObjectServerDebug.DEBUG_BLANK_STRING_KEY,
                String.valueOf(uri_count)));
    }

    public static String REFUSE_ENTRY = "oserv://127.0.0.1/,oserv://localhost/?netmask=0xff000000";
    public static String URI_HEAD = "URI ";
    

    public static void main(String[] args) {
        URITable  uri_table;
        URI       uri_entry;
        String    target;
        String    host_address;
        InetAddress inetAddress;

        try {
            uri_table = URITable.makeURITable(REFUSE_ENTRY);
        } catch (Exception exception_object) {
            exception_object.printStackTrace();
            return;
        }

        if (args.length == 0) {
            try {
                inetAddress = InetAddress.getLocalHost();
                target = inetAddress.getHostAddress();
            } catch (Exception exception_object) {
                exception_object.printStackTrace();
                return;
            }
        } else {
            target = args[0];
        }

        while(uri_table.hasMoreEntries()) {
            try {
                uri_entry = uri_table.nextEntry();
            } catch (Exception exception_object) {
                break;
            }
            System.out.println(uri_entry.toString());
        }
        uri_table.reset() ;
        while(uri_table.hasMoreEntries()) {
            try {
                uri_entry = uri_table.nextEntry();
            } catch (Exception exception_object) {
                break;
            }
            if (uri_table.isNetwork(uri_entry) == true) {
                if (uri_table.matchNetwork(uri_entry, target) == true) {
                }
            } else {
                host_address = uri_table.getHostAddress(uri_entry);
            }
            System.out.println(uri_entry.toString());
        }
    }
}
